#----------------------------------------------------------------------------
# Name:         ldmWidList.py
# Purpose:      ldmWidList.py
#               GUI widget respond on size change
# Author:       Walter Obweger
#
# Created:      20200405
# CVS-ID:       $Id$
# Copyright:    (c) 2020 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import sys
import wx

import six

from lindworm.ldmWidCore import ldmWidCore


typesIntType=type(1)
typesStringType=type('')
typesDictType=type({})
typesListType=type([])
typesTupleType=type(())

class ldmWidList(ldmWidCore):
    """list control widget shell to harmonize widget interface
    and ease usage.
    keyword arguments:
        lCol            ... column definition
        single_sel      ... single list item selection; 0|1
        multiple_sel    ... multiple list item selection; 0|1
    __initCfg__ modify widget configuration
        _bMultiple      ... list item selection mode; default=False
    """
    def __initCls__(self,**kwargs):
        """initialize class
        ### parameter
            kwargs  ... keyword arguments passed to all init methods
        """
        self.clsWid=wx.ListCtrl
    def __initCfg__(self,**kwargs):
        """initialize configuration
        ### parameter
            kwargs  ... keyword arguments passed to all init methods
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidCore::__initCfg__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            ldmWidCore.__initCfg__(self,**kwargs)
            # +++++ beg:
            self._bMultiple=False
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initObj__(self,**kwargs):
        """initialize object properties, widgets aren't present yet.
        data is supposed to be related to widgets or support their
        function.
        ### parameter
            kwargs  ... keyword arguments passed to all init methods
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidList::__initObj__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            self.IMG_EMPTY=-1
            self.iDat=1 #20200421 wro:py3 1L
            self.dDat={}
            if kwargs.get('multiple_sel',0)==0:
                self._bMultiple=False
            else:
                self._bMultiple=True
            if kwargs.get('single_sel',False)>0:
                self._bMultiple=False
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initWid__(self,**kwargs):
        """initialize widgets
        ### parameter
            kwargs  ... keyword arguments passed to all init methods
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidCore::__initWid__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            style=wx.LC_HRULES | wx.LC_REPORT | wx.LC_VRULES
            if self._bMultiple==False:
                style|=wx.LC_SINGLE_SEL
            _args,_kwargs=self.GetWidArgs(kwargs,
                        ['id','name','parent','pos','size','style'],
                        {'pos':(0,0),'size':(-1,-1),'style':style})
            self.wid=self.clsWid(*_args,**_kwargs)
            self._it=wx.ListItem()
            # ----- end:
            # +++++ beg:setup columns
            lCol=kwargs.get('lCol',None)
            if lCol is not None:
                for tCol in lCol:
                    if tCol[1] in ['l','lf']:
                        fmt=wx.LIST_FORMAT_LEFT
                    elif tCol[1] in ['r','rg']:
                        fmt=wx.LIST_FORMAT_RIGHT
                    elif tCol[1] in ['c','ct']:
                        fmt=wx.LIST_FORMAT_CENTER
                    else:
                        fmt=wx.LIST_FORMAT_LEFT
                    self.wid.AppendColumn(tCol[0],
                                    format=fmt,
                                    width=tCol[2])
            # ----- end:setup columns
        except:
            self.logTB()
        try:
            # +++++ beg:setup mapping
            oTmp=kwargs.get('map',None)
            if oTmp is not None:
                self.MAP=oTmp
            else:
                self.MAP=None
            # ----- end:setup mapping
            # +++++ beg:
            self._dImg=None
            self._imgLst=None
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def SetItem(self,idx,i,v):
        """set list column item
        ### parameter
            idx     ... 
            i       ... column
                    <0  ... data
                    >=0 ... column
            v       ... value
                    integer value, interpreted as image index
                    tuple, t[0] string value, t[1] image index
                    string or unicode value, image cleared
        """
        try:
            if self.IsMainThread()==False:
                return -2
            if i<0:
                i=self.wid.GetItemData(idx)
                i=self.__add_data__(v,i)
                self.wid.SetItemData(idx,i)
                return 0
            t=type(v)
            if t==typesTupleType:
                self.wid.SetStringItem(idx,i,v[0],v[1])
            elif t==typesIntType:
                self.wid.SetStringItem(idx,i,u'',v)
            else:
                self.wid.SetStringItem(idx,i,v,-1)
            return 0
        except:
            self.logTB()
    def GetItemTup(self,idx,i):
        """get item value
        ### parameter
            idx     ... row index
            i       ... column
                    -1  ... data
                    -3  ... data key
                    <0  ... None
                    >=0 ... column value as tuple, string and image index
        """
        try:
            if self.IsMainThread()==False:
                return None
            if i==-1:
                i=self.wid.GetItemData(idx)
                if i>0:
                    return self.dDat.get(i,None)
                return None
            elif i==-3:
                return self.wid.GetItemData(idx)
            elif i<0:
                return None
            it=self.wid.GetItem(idx,i)
            t=(it.GetText()[:],it.GetImage())
            return t
        except:
            self.logTB()
    def GetItemStr(self,idx,i):
        try:
            if self.IsMainThread()==False:
                return None
            if i==-1:
                i=self.wid.GetItemData(idx)
                if i>0:
                    return self.dDat.get(i,None)
                return None
            elif i==-3:
                return self.wid.GetItemData(idx)
            elif i<0:
                return None
            it=self.wid.GetItem(idx,i)
            return it.GetText()[:]
        except:
            self.logTB()
    def GetCount(self):
        """get row count
        """
        try:
            if self.IsMainThread()==False:
                return -1
            return self.wid.GetItemCount()
        except:
            self.logTB()
    def __add_data__(self,data,i=0):
        """add data to cache
        ### parameter
            data    ... object to cache
            i       ... key
                    0   ... use attribute iDat
        """
        try:
            if i==0:
                i=self.iDat
                self.iDat+=1
            self.dDat[i]=data
            return i
        finally:
            pass
        return -1
    def SetDictMap(self,lMap):
        """set MAP property to be used item set by dictionary 
        ### parameter
            lMap    ... mapping list of tuples keys to column number
                key 0   ... bitmap index (integer) or name (string)
                key -1  ... data to be cached and referenced to list item
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('SetDictMap')
            self.MAP=lMap
        except:
            self.logTB()
    def __setItemDict__(self,lst,idx,d):
        """set item by dictionary defined by property MAP
        ### parameter
            lst     ... widget
            idx     ... row
            d       ... dictionary values, property MAP used
        """
        #print tr.GetItemText(ti,0)
        for s,i in self.MAP:
            if s in d:
                v=d[s]
                t=type(v)
                if t==typesTupleType:
                    lst.SetStringItem(idx,i,v[0],v[1])
                elif t==typesIntType:
                    lst.SetStringItem(idx,i,u'',v)
                else:
                    lst.SetStringItem(idx,i,v,-1)
        if 0 in d:
            self._it.Clear()
            img=d[0]
            bSkipImg=True
            t=type(img)
            if t==typesIntType:
                self._it.SetImage(img)
                bSkipImg=False
            elif t==typesStringType:
                if self._dImg is not None:
                    self._it.SetImage(self._dImg[img])
                    bSkipImg=False
            if bSkipImg==False:
                self._it.SetMask(wx.LIST_MASK_IMAGE)
                self._it.SetId(idx)
                lst.SetItem(self._it)
                #lst.SetItem(idx,self._it)
        if -1 in d:
            i=lst.GetItemData(idx)
            i=self.__add_data__(d[-1],i)
            lst.SetItemData(idx,i)
        #if -2 in d:
        #    self.__setValue__(tr,ti,d[-2])
    def __setItemLst__(self,lst,idx,l):
        """set item by list
        ### parameter
            lst     ... widget
            idx     ... row
            l       ... list values, data and column values
        """
        try:
            if l[0] is not None:
                i=self.wid.GetItemData(idx)
                i=self.__add_data__(l[0],i)
                lst.SetItemData(idx,i)
            i=0
            for v in l[1]:
                t=type(v)
                if t==typesTupleType:
                    lst.SetItem(idx,i,v[1]) #v[0],
                elif t==typesIntType:
                    lst.SetItem(idx,i,u'')# v is image
                else:
                    lst.SetItem(idx,i,v)
                i+=1
        except:
            self.logTB()
            self.__logError__('l:%r;l[0]:%r;l[1]:%r'%(l,
                    l[0],l[1]))
    def __setItemTup__(self,tr,ti,t):
        """
        ### parameter
        """
        pass
    def __setItem__(self,lst,idx,val):
        """set item, various types are handled
        ### parameter
            lst     ... widget
            idx     ... row
            val     ... values, dictionary or list
        """
        t=type(val)
        if t==typesDictType:
            self.__setItemDict__(lst,idx,val)
        elif t==typesListType:
            self.__setItemLst__(lst,idx,val)
        elif t==typesTupleType:
            self.__setItemTup__(lst,idx,val)
    def __setValue__(self,lst,l,idx=-1):
        """set value
        ### parameter
            lst     ... widget
            l       ... list of values, dictionary or list
            idx     ... row
        """
        if idx<0:
            idx=0
        iCnt=lst.GetItemCount()
        for val in l:
            if idx>=iCnt:
                idx=lst.InsertItem(idx,'',self.IMG_EMPTY)
                iCnt+=1
            self.__setItem__(lst,idx,val)
            idx+=1
        return idx-1
    def __getValue__(self,lst,idx,lMap,l):
        """get item value
        ### parameter
            lst     ... widget
            idx     ... row
            lMap    ... list of columns to get
                None, delivers [-2]
                >=0 ... text of column
                -1  ... cached data
                -2  ... index
                -3  ... cache key
            l       ... list of values to return
        """
        def getVal(lst,idx,i):
            if i==-1:
                i=lst.GetItemData(idx)
                if i>0:
                    return self.dDat.get(i,None)
                return None
            elif i==-2:
                return idx
            elif i==-3:
                return lst.GetItemData(idx)
            else:
                it=lst.GetItem(idx,i)
                return it.GetText()[:]
        if lMap is None:
            l.append(getVal(lst,idx,-2))
        else:
            l.append([getVal(lst,idx,i) for i in lMap])
    def __getValueData__(self,lst,idx,l):
        """get value data
        ### parameter
            lst     ... widget
            idx     ... row
            l       ... list of values to return
        """
        i=lst.GetItemData(idx)
        if i>0:
            l.append(self.dDat.get(i,None))
    def __insert__(self,lst,l,idx=-1):
        """insert
        ### parameter
            lst     ... widget
            l       ... list of data and values to insert
            idx     ... row
                -1  ... insert at the end
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
            -1  ... exception
            -3  ... l is None
        """
        try:
            if l is None:
                return -3
            if idx<0:
                idx=lst.GetItemCount()
                for val in l:
                    idx=lst.InsertItem(idx+1,'')
                    self.__setItem__(lst,idx,val)
            else:
                idx-=1
                for val in l:
                    idx+=1
                    idx=lst.InsertItem(idx,'',self.IMG_EMPTY)
                    self.__setItem__(lst,idx,val)
            return idx
        except:
            self.logTB()
        return -1
    def Insert(self,l,idx=-1):
        """insert
        ### parameter
            l       ... list of data and values to insert
            idx     ... row
                -1  ... insert at the end
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
            -1  ... exception
            -2  ... not called from main thread
            -3  ... l is None
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('Insert')
            if self.IsMainThread()==False:
                return -2
            w=self.wid
            return self.__insert__(w,l,idx=idx)
        except:
            self.logTB()
        return -1
    def SetValue(self,l,idx=-1):
        """set value
        ### parameter
            l       ... list of data and values to set
            idx     ... row
                >=0 ... item index to start set, in case more
                        items are delivered to set, 
                -1  ... clear items
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
            -1  ... exception
            -2  ... not called from main thread
            -3  ... l is None
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('SetValue')
            if self.IsMainThread()==False:
                return -2
            w=self.wid
            #iCols=w.GetColumnCount()
            iRet=-2
            if idx==-1:
                w.DeleteAllItems()
                iRet=self.__insert__(w,l,idx=idx)
            else:
                iRet=self.__setValue__(w,l,idx=idx)
            #self.Sort()
            return iRet
        except:
            self.logTB()
        return -1
    def GetValue(self,lMap=None,idx=None):
        """get value
        ### parameter
            lMap    ... list of columns to get
                None .. cached data
                >=0 ... text of column
                -1  ... cached data
                -2  ... index
                -3  ... cache key
            idx     ... row
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('GetValue')
            if self.IsMainThread()==False:
                return None
            l=[]
            lst=self.wid
            if idx is None:
                idx=0
                iCnt=-1
            elif idx<0:
                idx=0
                iCnt=-1
            else:
                iCnt=1
            if iCnt<0:
                iCnt=lst.GetItemCount()
            if lMap is None:
                for i in six.moves.xrange(idx,idx+iCnt):
                    self.__getValueData__(lst,i,l)
            else:
                for i in six.moves.xrange(idx,idx+iCnt):
                    self.__getValue__(lst,i,lMap,l)
            if bDbg:
                self.logDbg('GetValue:%r',l)
            return l
        except:
            self.logTB()
        return None
    def PrcFct(self,idx,bDbg,fct,*args,**kwargs):
        """process function on item
        ### parameter
            idx     ... index
            bDbg    ... debugging flag
            fct     ... callable applied on each item in addition
                        to args and kwargs following values are passed
                iOfs .. current processing number starting at 0
                iCnt .. total number of calls
                iIdx .. item index
            args    ... arguments
            kwargs  ... keyword arguments
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('PrcFct iIdx:%r args:%r kwargs:%r',idx,args,kwargs)
            lst=self.wid
            if idx is None:
                idx=0
                iCnt=-1
            elif idx<0:
                idx=0
                iCnt=-1
            else:
                iCnt=1
            if iCnt<0:
                iCnt=lst.GetItemCount()
            if bDbg:
                self.logDbg('iCnt:%d iIdx:%d',iCnt,idx)
            iOfs=0
            for i in six.moves.xrange(idx,idx+iCnt):
                if bDbg:
                    self.logDbg('iOfs:%d-%d iIdx:%d',iOfs,iCnt,i)
                iRet=fct(iOfs,iCnt,i,*args,**kwargs)
                if iRet<=0:
                    return iRet
                iOfs+=1
            return 1
        except:
            self.logTB()
            return -1
    def __setState__(self,lst,l,bFlag,iState):
        """set state
        ### parameter
            lst     ... widget
            l       ... list of index
            bFlag   ... flag
            iState  ... item status
        """
        try:
            iCnt=lst.GetItemCount()
            iIdxFirst=-1
            if bFlag==True:
                if l is None:
                    pass
                else:
                    if self._bMultiple==False:      # 20150727 wro:single sel 
                        lst.SetItemState(l[0],iState,iState)
                        return l[0]
                    for i in l:
                        lst.SetItemState(i,iState,iState)
                        if iIdxFirst<0:
                            iIdxFirst=i
                    return iIdxFirst
            else:
                if l is None:
                    for i in six.moves.xrange(iCnt):
                        lst.SetItemState(i,0,iState)
                        if iIdxFirst<0:
                            iIdxFirst=i
                    return iIdxFirst
                else:
                    for i in l:
                        lst.SetItemState(i,0,iState)
                        if iIdxFirst<0:
                            iIdxFirst=i
                    return iIdxFirst
            return 0
        except:
            self.logTB()
        return -1
    def __setSelected__(self,lst,l,bFlag):
        """set selected status
        ### parameter
            lst     ... widget
            l       ... list of index
            bFlag   ... flag
        """
        return self.__setState__(lst,l,bFlag,wx.LIST_STATE_SELECTED)
    def __getState__(self,lst,lMap,l,iState):
        """get item values with state set
        ### parameter
            lst     ... widget
            lMap    ... list of columns to get
                None, delivers [-2]
                >=0 ... text of column
                -1  ... cached data
                -2  ... index
                -3  ... cache key
            l       ... list of values to return
            iState  ... state
        ### return
            number of items matching state
        """
        try:
            iCnt=lst.GetItemCount()
            iSel=0
            for i in six.moves.xrange(0,iCnt):
                if (lst.GetItemState(i,iState)&iState)==iState:
                    self.__getValue__(lst,i,lMap,l)
                    iSel+=1
            return iSel
        except:
            self.logTB()
        return -1
    def __getSelected__(self,lst,lMap,l):
        """get selected item values
        ### parameter
            lst     ... widget
            lMap    ... list of columns to get
                None, delivers [-2]
                >=0 ... text of column
                -1  ... cached data
                -2  ... index
                -3  ... cache key
            l       ... list of values to return
        ### return
            number of items matching state
        """
        return self.__getState__(lst,lMap,l,wx.LIST_STATE_SELECTED)
    def SetSelected(self,l,bFlag=True,lMapChk=None,bKeepSel=False,bEnsure=True):
        """set selected
        ### parameter
            l       ... dictionary

            bFlag   ... flag, True or False
            lMapChk ... mapping 
                if check is None, and l is dictionary lMapChk is constructed
                    integer keys are added
                    keys found in MAP property
            bKeepSel .. keep selected
            bEnsure ... ensure first selected to be visible
        """
        try:
            bDbg=self.GetVerboseDbg(0)
            if bDbg:
                self.logDbg('SetSelected')
            bDb0=self.GetVerboseDbg(0)
            bDbg=self.GetVerboseDbg(10)
            if self.IsMainThread()==False:
                return -2
            lst=self.wid
            if self._bMultiple==True:
                if bKeepSel==False:
                    lSel=[]
                    self.__getSelected__(lst,None,lSel)
                    self.__setSelected__(lst,lSel,bFlag=False)
            t=type(l)
            if t==typesDictType:
                if lMapChk is None:
                    lKeys=l.keys()
                    lMapChk=[]
                    ll=[]
                    for sK in lKeys:
                        if type(sK)==typesIntType:
                            lMapChk.append(sK)
                            ll.append(l[sK])
                        else:
                            try:
                                bFound=False
                                for tM in self.MAP:
                                    if tM[0]==sK:
                                        lMapChk.append(tM[1])
                                        bFound=True
                                        break
                                if bFound==True:
                                    ll.append(l[sK])
                                else:
                                    self.logErr('mapping faulty %r',
                                            {'sK':sK,'MAP':self.MAP})
                            except:
                                self.logTB()
                    l=[ll]
                    if bDb0:
                        self.logDbg('%r',{'lMapChk':lMapChk})
            if lMapChk is None:
                idx=self.__setSelected__(lst,l,bFlag=bFlag)
            else:
                iCntChk=len(lMapChk)
                iCnt=lst.GetItemCount()
                if iCntChk==1:
                    if iCnt>0:
                        t=type(l[0])
                        if t==typesListType:
                            pass
                        elif t==typesListType:
                            pass
                        else:   # 20150118 wro:values need to be list,
                                #       convert l from [1,2] to l=[[1],[2]]
                            ll=[]
                            for v in l:
                                ll.append([v])
                            l=ll
                lIdx=[]
                
                for i in six.moves.xrange(iCnt):
                    ll=[]
                    self.__getValue__(lst,i,lMapChk,ll)
                    for val in l:
                        if bDbg:
                            self.logDbg('%r',{'ll':ll,'val':val})
                        if ll[0]==val:
                            
                            if bDbg:
                                self.logDbg('%r',{'ll[0]':ll[0],'val':val})
                            lIdx.append(i)
                if bDb0:
                    self.logDbg('%r',lIdx)
                idx=self.__setSelected__(lst,lIdx,bFlag=bFlag)
            if bEnsure==True:
                if idx>=0:
                    lst.EnsureVisible(idx)
            return idx
        except:
            self.logTB()
        return -1
    def GetSelected(self,lMap=None):
        """get selected elements
        ### parameter
            lMap    ... list of column offsets to retrieve
                        None will deliver [-2,-1]
                -3  ... cache key
                -2  ... index
                -1  ... cached data
                >=0 ... text of column
        ### return
            list of selected items, holding mapped values
        """
        try:
            self.logDbg('GetSelected lMap:%r',lMap)
            if self.IsMainThread()==False:
                return
            l=[]
            lst=self.wid
            self.__getSelected__(lst,lMap or [-2,-1],l)
            return l
        except:
            self.logTB()
        return None
    def DelIdx(self,iIdx):
        """delete index
        ### parameter
            iIdx    ... item to delete form list
        """
        try:
            self.wid.DeleteItem(iIdx)
        except:
            self.logTB()
    def DelSelected(self):
        """delete selected
        ### parameter
        """
        try:
            lSel=self.GetSelected([-2,-1])
            self.logDbg('DelSelected lSel:%r',lSel)
            lSel.reverse()
            for t in lSel:
                self.DelIdx(t[0])
        except:
            self.logTB()



#----------------------------------------------------------------------------
# Name:         ldmWidTree.py
# Purpose:      ldmWidTree.py
#               GUI widget respond on size change
# Author:       Walter Obweger
#
# Created:      20200405
# CVS-ID:       $Id$
# Copyright:    (c) 2020 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import wx
from lindworm.ldmWidCore import ldmWidCore

class ldmWidTree(ldmWidCore):
    def __initCls__(self,**kwargs):
        self.clsWid=wx.TreeCtrl
    def __initWid__(self,**kwargs):
        try:
            self.logDbg('__initWid__')
            style=0
            self.IMG_EMPTY=-1
            if kwargs.get('bTreeButtons',1)>0:
                style|=wx.TR_HAS_BUTTONS
            if kwargs.get('bHideRoot',False)>0:
                style|=wx.TR_HIDE_ROOT
            if kwargs.get('multiple_sel',False)>0:
                style|=wx.TR_MULTIPLE
                self._bMultiple=True
            else:
                self._bMultiple=False
            _args,_kwargs=self.GetWidArgs(kwargs,
                        ['id','name','parent','pos','size','style'],
                        {'pos':(0,0),'size':(-1,-1),'style':style})
            self.wid=self.clsWid(*_args,**_kwargs)
        except:
            self.logTB()



#----------------------------------------------------------------------------
# Name:         ldmStorageLine.py
# Purpose:      storage for lines
#
# Author:       Walter Obweger
#
# Created:      20200104
# CVS-ID:       $Id$
# Copyright:    Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import logging
import traceback

from lindworm.ldmStorage import ldmStorage

class ldmStorageLine(ldmStorage):
    def __init__(self,iModeRev=0,sLogger='',iLv=1,iVerbose=0):
        """constructor
        ### parameter
            iModeRev    ... reverse processing
                    1   ... FILO, first in last out
                    0   ... FIFO, first in first out
            sLogger     ... log origin
            iLv         ... logging level
            iVerbose    ... higher values add more logs
        """
        ldmStorage.__init__(self,iModeRev=iModeRev,
                            sLogger=sLogger,iLv=iLv,
                            iVerbose=iVerbose)
        #self.__initDat__()
    def __initPrc__(self):
        """initialize data properties
        """
        try:
            ldmStorage.__initPrc__(self)
            # +++++ beg:initialize
            sOrg='ldmStorageFolder::__initDat__'
            self.oLog.debug('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:initialize data
            self.sEnc=None
            self.lLine=[]
            self.iCnt=0
            self.iAct=-1
            self.lMarker=[]
            # ----- end:initialize data
            self.oLog.debug('end:%s'%(sOrg))
        except:
            self.logTB()
            return -1
    def prcBeg(self,sDef,oRef=None,**kwargs):
        """processing begin, add file name and objects to stack
        ### parameter
            sFN     ... file name : str
            oDat    ... data object
            oRef    ... reference object
            kwargs  ... flexible keyword argument
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmStorageLine::prcBeg'
            # ----- end:initialize
            # +++++ beg:processing begin
            if self.iVerbose>0:
                self.oLog.debug('beg:%s sDef:%s'%(sOrg,sDef))
            dRef={
                'iAct':self.iAct,
                'oRef':oRef,
            }
            iRet=ldmStorage.prcBeg(self,sDef,oRef=dRef,**kwargs)
            if iRet>0:
                self.iAct=0
            if iRet>=0:
                self.lMarker.append([sDef,self.iCnt,self.iAct])
            if self.iVerbose>0:
                self.oLog.debug('end:%s iRet:%d'%(sOrg,iRet))
            # ----- end:processing begin
            return iRet
        except:
            self.logTB()
            return -1
    def prcExc(self,**kwargs):
        """processing execution
        ### parameter
            **kwargs ... flexible keyword argument
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmStorageLine::prcExc'
            # ----- end:initialize
            # +++++ beg:
            iRet,sDef=self.getDef()
            if iRet<1:
                self.oLog.debug('skp:%s iRet:%d empty stack'%('ldmStorageLine::prcExc',iRet))
                return iRet
            self.oLog.debug('beg:%s sDef:%s'%(sOrg,sDef))
            if self.iVerbose>0:
                self.oLog.debug('    kwargs:%r'%(kwargs))
            iLine=0
            with open(sDef,'r',encoding=self.sEnc) as oFileIn:
                iLine=1
                for sLine in oFileIn:
                    # +++++ beg:process line
                    if self.iVerbose>50:
                        self.oLog.debug('  iLine:%04d sLine:>%s<'%(iLine,sLine))
                    iR,sL=self.prcLine(iLine,sLine,**kwargs)
                    if self.iVerbose>50:
                        self.oLog.debug('  iLine:%04d iR:%d sL:>%s<'%(iLine,iR,sL))
                    if iR>0:
                        self.addLine(sL)
                    iLine=iLine+1
                    # ----- end:process line
            self.oLog.debug('end:%s iRet:%d'%(sOrg,iRet))
            # ----- end:
            return iRet
        except:
            self.logTB()
            return -1
    def prcEnd(self,**kwargs):
        """processing end
        ### parameter
            **kwargs ... flexible keyword argument
            
            use updated properties to access current properties to be finalized
            self.sDefEnd    ... definition
            self.oArgEnd    ... keyword arguments passed at prcBeg
            self.oDatEnd    ... data object
            self.oRefEnd    ... reference object
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmStorageLine::prcEnd'
            # ----- end:initialize
            # +++++ beg:processing end
            if self.iVerbose>0:
                self.oLog.debug('beg:%s len(lMarker):%d'%(sOrg,
                                len(self.lMarker)))
            iRet=ldmStorage.prcEnd(self,**kwargs)
            if iRet>0:
                #sMarkerFN,iMarkerCnt,iMarkerAct=self.lMarker[-1]
                #del self.lMarker[-1]
                #if sFN!=sMarkerFN:
                #    self.oLog.error('    %s sFN:%s sMarkerFN:%s mismatch'%('ldmStorageLine::prcEnd',
                #                    sFN,sMarkerFN))
                # +++++ beg:revert current file info
                #self.iAct=iMarkerAct    # revert current file line back
                # ----- end:revert current file info
                if self.iVerbose>5:
                    self.oLog.debug('    %s iAct:%5d iCnt:%6d sDefEnd:%s'%('ldmStorageLine::prcEnd',
                                    self.iAct,self.iCnt,self.sDefEnd))
            if self.iVerbose>0:
                self.oLog.debug('end:%s iRet:%d'%(sOrg,iRet))
            # ----- end:processing end
            return iRet
        except:
            self.logTB()
            return -1
    def addLine(self,sLine):
        """add line to storage
        ### parameter
            sLine   ... line to add, including line feed character(s)
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            iRet=0
            # ----- end:initialize
            # +++++ beg:add line to storage
            if sLine is not None:
                self.iAct=self.iAct+1
                self.lLine.append((self.iAct,sLine))
                self.iCnt=self.iCnt+1
                iRet=1
            if self.iVerbose>15:
                self.oLog.debug('%s iRet:%d iAct:%5d %-20s'%('ldmStorageLine::addLine',
                                iRet,self.iAct,sLine[:18]))
            # ----- end:add line to storage
            return iRet
        except:
            self.logTB()
            return -1
    def getLines(self):
        """add line to storage
        ### return
            lLine   ... list lines
            iCnt    ... line count
        """
        return self.lLine,self.iCnt
    def prcLine(self,iLine,sLine,**kwargs):
        """process line
        ### parameter
            iLine   ... line number : int
            sLine   ... line : str to process, including line feed character(s)
            **kwargs ... flexible keyword argument
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmStorageLine::prcLine'
            # ----- end:initialize
            # +++++ beg:process line
            if self.iVerbose>10:
                self.oLog.debug('beg:%s len(sLine):%d'%(sOrg,
                                iLine,len(sLine)))
            iRet=1
            if self.iVerbose>10:
                self.oLog.debug('end:%s iRet:%d'%(sOrg,iRet))
            # ----- end:process line
            return iRet,sLine
        except:
            self.logTB()
            return -1,None



#----------------------------------------------------------------------------
# Name:         ldmWidFrmAui.py
# Purpose:      ldmWidFrmAui.py
#               frame widget utilizing advanced user interface
# Author:       Walter Obweger
#
# Created:      20200405
# CVS-ID:       $Id$
# Copyright:    (c) 2020 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import wx
import wx.aui

from lindworm.ldmArg import ldmArg
from lindworm.ldmWidCore import ldmWidCore
import lindworm.ldmWidImgMed as ldmWidImgMed
from lindworm.ldmWidSizeRspWid import ldmWidSizeRspWid
from lindworm.ldmWidList import ldmWidList
from lindworm.ldmWidTree import ldmWidTree

#class ldmAuiMpw(aui.AuiMDIParentFrame):
class ldmWidFrmAui(ldmWidCore):
    ID_TBR_File=100
    ID_FILE_OPEN=101
    ID_FILE_SAVE=102
    ID_File_EXIT=190
    ID_EDIT_CUT=201
    ID_EDIT_COPY=202
    ID_EDIT_PASTE=203
    ID_CMD=300
    ID_DST=700
    def __initCls__(self,**kwargs):
        self.clsWid=wx.Frame
    def __initCfg__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initCfg__'
            self.logDbg('beg:%s'%(sOrg))
            ldmWidCore.__initCfg__(self,**kwargs)
            # ----- end:initialize
            # +++++ beg:
            self.tTbrBmpSz=wx.Size(32, 32)
            # ----- end:
            # +++++ beg:
            self.setCfgWid('toolbar','bmp.iSz','32')
            # ----- end:
            # +++++ beg:toolbar file
            self.setCfgWid('tbrFile','enable','1')
            self.setCfgWid('tbrFile','open.tip','Open Json File')
            self.setCfgWid('tbrFile','save.tip','Save Json File')
            # ----- end:toolbar file
            # +++++ beg:toolbar edit
            self.setCfgWid('tbrEdit','enable','1')
            self.setCfgWid('tbrEdit','cut.tip','cut')
            self.setCfgWid('tbrEdit','copy.tip','copy')
            self.setCfgWid('tbrEdit','paste.tip','paste')
            # ----- end:toolbar edit
            # +++++ beg:toolbar command
            self.setCfgWid('toolbar','command','1')
            self.setCfgWid('tbrCmd','00.enable','1')
            self.setCfgWid('tbrCmd','01.enable','1')
            self.setCfgWid('tbrCmd','02.enable','1')
            self.setCfgWid('tbrCmd','03.enable','1')
            self.setCfgWid('tbrCmd','04.enable','0')
            self.setCfgWid('tbrCmd','05.enable','0')
            self.setCfgWid('tbrCmd','06.enable','0')
            self.setCfgWid('tbrCmd','07.enable','0')
            self.setCfgWid('tbrCmd','08.enable','0')
            self.setCfgWid('tbrCmd','09.enable','0')
            self.setCfgWid('tbrCmd','00.tip','command 00')
            self.setCfgWid('tbrCmd','01.tip','command 01')
            self.setCfgWid('tbrCmd','02.tip','command 02')
            self.setCfgWid('tbrCmd','03.tip','command 03')
            self.setCfgWid('tbrCmd','04.tip','command 04')
            self.setCfgWid('tbrCmd','05.tip','command 05')
            self.setCfgWid('tbrCmd','06.tip','command 06')
            self.setCfgWid('tbrCmd','07.tip','command 07')
            self.setCfgWid('tbrCmd','08.tip','command 08')
            self.setCfgWid('tbrCmd','09.tip','command 09')
            # ----- end:toolbar command
            # +++++ beg:menubar command
            self.setCfgWid('menubar','command','1')
            self.setCfgWid('mnrCmd','00.enable','1')
            self.setCfgWid('mnrCmd','01.enable','1')
            self.setCfgWid('mnrCmd','02.enable','1')
            self.setCfgWid('mnrCmd','03.enable','1')
            self.setCfgWid('mnrCmd','04.enable','0')
            self.setCfgWid('mnrCmd','05.enable','0')
            self.setCfgWid('mnrCmd','06.enable','0')
            self.setCfgWid('mnrCmd','07.enable','0')
            self.setCfgWid('mnrCmd','08.enable','0')
            self.setCfgWid('mnrCmd','09.enable','0')
            self.setCfgWid('mnrCmd','00.menu','command 00')
            self.setCfgWid('mnrCmd','01.menu','command 01')
            self.setCfgWid('mnrCmd','02.menu','command 02')
            self.setCfgWid('mnrCmd','03.menu','command 03')
            self.setCfgWid('mnrCmd','04.menu','command 04')
            self.setCfgWid('mnrCmd','05.menu','command 05')
            self.setCfgWid('mnrCmd','06.menu','command 06')
            self.setCfgWid('mnrCmd','07.menu','command 07')
            self.setCfgWid('mnrCmd','08.menu','command 08')
            self.setCfgWid('mnrCmd','09.menu','command 09')
            # ----- end:toolbar command
            # +++++ beg:toolbar destination
            self.setCfgWid('toolbar','destination','1')
            # ----- end:toolbar destination
            # +++++ beg:
            self.setCfgWid('frmMain','sToolBarCommand','0')
            self.setCfgWid('frmMain','sToolBarDestination','0')
            #dCfgFrm={}
            #self.dCfgWidDft['frmMain']=dCfgFrm
            #dCfgFrm['sToolBarCommand']='0'
            #dCfgFrm['sToolBarDestination']='0'
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initObj__(self,**kwargs):
        """initialize object properties, widgets aren't present yet.
        data is supposed to be related to widgets or support their
        function.
        ### parameter
            kwargs  ... keyword arguments passed to all init methods
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidCore::__initObj__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            ldmWidCore.__initObj__(self,**kwargs)
            # ----- end:
            # +++++ beg:
            iSz=self.getCfgWid('toolbar','bmp.iSz',sType='int',oDft='32')
            self.tTbrBmpSz=wx.Size(iSz, iSz)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initWid__(self,**kwargs):
        """initialize widget
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:
            # ----- end:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initWid__'
            self.logDbg('beg:%s'%(sOrg))
            tSz=(
                    self.getCfgWid('frmMain','width',sType='int',oDft=640),
                    self.getCfgWid('frmMain','height',sType='int',oDft=480)
                    )
            style=wx.DEFAULT_FRAME_STYLE
            # ----- end:initialize
            self.logDbg('kw:%r',kwargs)
            lArg=['parent','id','title','pos','size','style']
            _args,_kwargs=self.GetWidArgs(kwargs,
                        ['id','name','parent','size','style','title'],
                        {'size':tSz,'style':style},
                        lArg=lArg)
            #self.logDbg('arg:%r',_args)
            #self.logDbg('kw:%r',_kwargs)
            self.wid=self.clsWid(*_args,**_kwargs)
            self.oApi=wx.aui.AuiPaneInfo()
            self.mgrMain=wx.aui.AuiManager()
            self.mgrMain.SetManagedWindow(self.wid)
            # set frame icon
            #self.SetIcon(images.Mondrian.GetIcon())

            #mb = self.MakeMenuBar()
            #self.SetMenuBar(mb)
            self.__initMenuBar__(**kwargs)
            self.__initStatusBar__(**kwargs)
            self.__initToolBar__(**kwargs)
            # +++++ beg:create center panel
            self.oApi.Name("nbCenter")
            self.oApi.CenterPane()
            self.oApi.PaneBorder(False)
            self.oApi.Center()
            wNb=self.__initPanCt__(**kwargs)
            # +++++ beg:create help panel
            sLbl=self.getCfgWid('help','tab',oDft='help')
            if len(sLbl)>0:
                wid=wx.html.HtmlWindow(wNb, -1, 
                                    wx.DefaultPosition,
                                    wx.Size(400, 300))
                sHlp=self.getCfgWid('help','text',oDft="<html><body>"\
                            '<h3>FIXME</h3>'\
                            "<b>configure widget in json file</b></br>"\
                            "<ul>" \
                            "<li>help->tab:tabulator label</li>"\
                            "<li>help->text:help text</li>"\
                            "</ul>" \
                            "</body></html>")
                wid.SetPage(sHlp)
                wNb.AddPage(wid,sLbl)
                pass
            # ----- end:create help panel
            # +++++ beg:create CLI argument panel
            oArg=kwargs.get('oArg',None)
            if oArg is not None:
                wCliArg=oArg.__initPan__(self,wNb)
                self.logDbg('    args init')
            else:
                self.logDbg('    args none found')
            # ----- end:create CLI argument panel
            # ----- end:create center panel
            self.oApi.Left()
            self.__initPanLf__(**kwargs)
            self.oApi.Bottom()
            self.__initPanBt__(**kwargs)
            self.mgrMain.Update()
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initToolBar__(self,**kwargs):
        """initialize toolbars
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initToolBar__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            self.__initTbrFile__(**kwargs)
            self.__initTbrEdit__(**kwargs)
            iVal=self.getCfgWid('toolbar','command',sType='int',oDft=0)
            if iVal>0:
                self.__initTbrCmd__(**kwargs)
            iVal=self.getCfgWid('toolbar','destination',sType='int',oDft=0)
            if iVal>0:
                self.__initTbrDst__(**kwargs)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initTbrFile__(self,**kwargs):
        """initialize toolbar file
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initTbrFile__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            oSz=self.tTbrBmpSz
            iStyle =wx.aui.AUI_TB_DEFAULT_STYLE
            #iStyle|=wx.aui.AUI_TB_OVERFLOW
            tbr = wx.aui.AuiToolBar(self.GetWid(),
                        -1,
                        wx.DefaultPosition,
                        size=oSz or wx.DefaultSize,
                        style=iStyle)
            
            bmp=wx.ArtProvider.GetBitmap(wx.ART_FILE_OPEN,size=oSz)
            sTip=self.getCfgWid('tbrFile','open.tip',oDft='open json file')
            w=tbr.AddTool(self.ID_FILE_OPEN,
                        label="Open",
                        bitmap=bmp,
                        kind=wx.ITEM_NORMAL,
                        short_help_string=sTip)
            self.BindEvent('mn',self.OnFileOpen,w)
            bmp=wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE,size=oSz)
            sTip=self.getCfgWid('tbrFile','save.tip',oDft='save json file')
            w=tbr.AddTool(self.ID_FILE_SAVE,
                        label="Save",
                        bitmap=bmp,
                        kind=wx.ITEM_NORMAL,
                        short_help_string=sTip)
            self.BindEvent('mn',self.OnFileSave,w)

            oApi=wx.aui.AuiPaneInfo().Name('tbrFile')
            oApi.Caption('file toolbar')
            oApi.ToolbarPane().Top()
            self.mgrMain.AddPane(tbr,oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initTbrEdit__(self,**kwargs):
        """initialize toolbar edit
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initTbrEdit__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            oSz=self.tTbrBmpSz
            iStyle =wx.aui.AUI_TB_DEFAULT_STYLE
            #iStyle|=wx.aui.AUI_TB_OVERFLOW
            tbr = wx.aui.AuiToolBar(self.GetWid(),
                        -1,
                        wx.DefaultPosition,
                        size=oSz or wx.DefaultSize,
                        style=iStyle)
            
            bmp=wx.ArtProvider.GetBitmap(wx.ART_CUT,size=oSz)
            sTip=self.getCfgWid('tbrEdit','cut.tip',oDft='cut')
            w=tbr.AddTool(self.ID_EDIT_CUT,
                        label="Cut",
                        bitmap=bmp,
                        kind=wx.ITEM_NORMAL,
                        short_help_string=sTip)
            self.BindEvent('mn',self.OnEditCut,w)
            bmp=wx.ArtProvider.GetBitmap(wx.ART_COPY,size=oSz)
            sTip=self.getCfgWid('tbrEdit','copy.tip',oDft='copy')
            w=tbr.AddTool(self.ID_EDIT_COPY,
                        label="Copy",
                        bitmap=bmp,
                        kind=wx.ITEM_NORMAL,
                        short_help_string=sTip)
            self.BindEvent('mn',self.OnEditCopy,w)
            bmp=wx.ArtProvider.GetBitmap(wx.ART_PASTE,size=oSz)
            sTip=self.getCfgWid('tbrEdit','paste.tip',oDft='paste')
            w=tbr.AddTool(self.ID_EDIT_PASTE,
                        label="Paste",
                        bitmap=bmp,
                        kind=wx.ITEM_NORMAL,
                        short_help_string=sTip)
            self.BindEvent('mn',self.OnEditPaste,w)
            
            oApi=wx.aui.AuiPaneInfo().Name('tbrEdit')
            oApi.Caption('edit toolbar')
            oApi.ToolbarPane().Top()
            self.mgrMain.AddPane(tbr,oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initTbrCmd__(self,**kwargs):
        """initialize toolbar command
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initTbrDst__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            oSz=self.tTbrBmpSz
            iStyle =wx.aui.AUI_TB_DEFAULT_STYLE
            tbr = wx.aui.AuiToolBar(self.GetWid(),
                        -1,
                        wx.DefaultPosition,
                        size=oSz or wx.DefaultSize,
                        style=iStyle)
            #tbr.SetToolBitmapSize(oSz)
            
            iEn=self.getCfgWid('tbrCmd','00.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn00.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','00.tip',oDft='command 00')
                w=tbr.AddTool(self.ID_CMD+0,
                            label="00",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd00,w)
            
            iEn=self.getCfgWid('tbrCmd','01.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn01.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','01.tip',oDft='command 01')
                w=tbr.AddTool(self.ID_CMD+1,
                            label="01",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd01,w)
            
            iEn=self.getCfgWid('tbrCmd','02.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn02.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','02.tip',oDft='command 02')
                w=tbr.AddTool(self.ID_CMD+2,
                            label="02",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd02,w)
            
            iEn=self.getCfgWid('tbrCmd','03.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn03.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','03.tip',oDft='command 03')
                w=tbr.AddTool(self.ID_CMD+3,
                            label="01",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd03,w)
            
            iEn=self.getCfgWid('tbrCmd','04.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn04.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','04.tip',oDft='command 04')
                w=tbr.AddTool(self.ID_CMD+4,
                            label="04",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd04,w)
            
            iEn=self.getCfgWid('tbrCmd','05.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn05.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','05.tip',oDft='command 05')
                w=tbr.AddTool(self.ID_CMD+5,
                            label="05",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd05,w)
            
            iEn=self.getCfgWid('tbrCmd','06.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn06.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','06.tip',oDft='command 06')
                w=tbr.AddTool(self.ID_CMD+6,
                            label="06",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd06,w)
            
            iEn=self.getCfgWid('tbrCmd','07.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn07.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','07.tip',oDft='command 07')
                w=tbr.AddTool(self.ID_CMD+7,
                            label="07",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd07,w)
            
            iEn=self.getCfgWid('tbrCmd','08.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn08.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','08.tip',oDft='command 08')
                w=tbr.AddTool(self.ID_CMD+8,
                            label="08",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd08,w)
            
            iEn=self.getCfgWid('tbrCmd','09.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn09.GetBitmap()
                sTip=self.getCfgWid('tbrCmd','09.tip',oDft='command 09')
                w=tbr.AddTool(self.ID_CMD+9,
                            label="09",
                            bitmap=bmp,
                            kind=wx.ITEM_NORMAL,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnCmd09,w)
            
            oApi=wx.aui.AuiPaneInfo().Name('tbrCmd')
            oApi.Caption('command toolbar')
            oApi.ToolbarPane().Top()
            self.mgrMain.AddPane(tbr,oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initTbrDst__(self,**kwargs):
        """initialize toolbar destination
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initTbrDst__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            oSz=self.tTbrBmpSz
            iStyle =wx.aui.AUI_TB_DEFAULT_STYLE
            iStyle|=wx.aui.AUI_TB_VERTICAL
            tbr = wx.aui.AuiToolBar(self.GetWid(),
                        -1,
                        wx.DefaultPosition,
                        size=oSz or wx.DefaultSize,
                        style=iStyle)
            #tbr.SetToolBitmapSize(oSz)
            iOfs=0
            self.lWidDstChk=[None]*10
            
            iEn=self.getCfgWid('tbrDst','00.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn00.GetBitmap()
                sTip=self.getCfgWid('tbrDst','00.tip',oDft='destination 00')
                w=tbr.AddTool(self.ID_DST+0,
                            label="00",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst00,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','01.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn01.GetBitmap()
                sTip=self.getCfgWid('tbrDst','01.tip',oDft='destination 01')
                w=tbr.AddTool(self.ID_DST+1,
                            label="01",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst01,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','02.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn02.GetBitmap()
                sTip=self.getCfgWid('tbrDst','02.tip',oDft='destination 02')
                w=tbr.AddTool(self.ID_DST+2,
                            label="02",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst02,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','03.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn03.GetBitmap()
                sTip=self.getCfgWid('tbrDst','03.tip',oDft='destination 03')
                w=tbr.AddTool(self.ID_DST+3,
                            label="03",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst03,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','04.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn04.GetBitmap()
                sTip=self.getCfgWid('tbrDst','04.tip',oDft='destination 04')
                w=tbr.AddTool(self.ID_DST+4,
                            label="04",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst04,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','05.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn05.GetBitmap()
                sTip=self.getCfgWid('tbrDst','05.tip',oDft='destination 05')
                w=tbr.AddTool(self.ID_DST+5,
                            label="05",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst05,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','06.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn06.GetBitmap()
                sTip=self.getCfgWid('tbrDst','01.tip',oDft='destination 06')
                w=tbr.AddTool(self.ID_DST+6,
                            label="06",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst06,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','07.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn07.GetBitmap()
                sTip=self.getCfgWid('tbrDst','07.tip',oDft='destination 07')
                w=tbr.AddTool(self.ID_DST+7,
                            label="07",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst01,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','08.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn08.GetBitmap()
                sTip=self.getCfgWid('tbrDst','08.tip',oDft='destination 08')
                w=tbr.AddTool(self.ID_DST+8,
                            label="08",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst08,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iEn=self.getCfgWid('tbrDst','09.enable',sType='int',oDft=1)
            if iEn>0:
                bmp=ldmWidImgMed.Gn09.GetBitmap()
                sTip=self.getCfgWid('tbrDst','09.tip',oDft='destination 09')
                w=tbr.AddTool(self.ID_DST+9,
                            label="09",
                            bitmap=bmp,
                            kind=wx.ITEM_RADIO,
                            short_help_string=sTip)
                self.BindEvent('mn',self.OnDst09,w)
                self.lWidDstChk[iOfs]=w
            iOfs+=1
            
            iChecked=self.getCfgWid('tbrDst','checked',sType='int',oDft=-1)
            try:
                w=self.lWidDstChk[iChecked]
                #AUI_BUTTON_STATE_NORMAL
                #AUI_BUTTON_STATE_HOVER
                #AUI_BUTTON_STATE_PRESSED
                #AUI_BUTTON_STATE_DISABLED
                #AUI_BUTTON_STATE_HIDDEN
                w.SetState(wx.aui.AUI_BUTTON_STATE_CHECKED)
            except:
                self.logTB()
                self.logErr('iChecked:%d faulty',iChecked)

            oApi=wx.aui.AuiPaneInfo().Name('tbrDst')
            oApi.Caption('destination toolbar')
            oApi.ToolbarPane().Right()
            oApi.GripperTop().TopDockable(False).BottomDockable(False)
            self.mgrMain.AddPane(tbr,oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initPanCt__(self,**kwargs):
        """initialize pane center
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initPanCt__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            iStyle =wx.aui.AUI_NB_TOP | wx.aui.AUI_NB_TAB_SPLIT 
            iStyle|=wx.aui.AUI_NB_SCROLL_BUTTONS
            #iStyle|=wx.aui.AUI_NB_TAB_MOVE
            #iStyle|=wx.aui.AUI_NB_CLOSE_ON_ACTIVE_TAB
            #iStyle|=wx.aui.AUI_NB_MIDDLE_CLICK_CLOSE
            wNb=wx.aui.AuiNotebook(self.GetWid(), -1, 
                        (0,0),
                        wx.Size(430, 200),
                        style=iStyle)
            self.mgrMain.AddPane(wNb,self.oApi)
            self.oApi.Name("nbCenter")
            self.oApi.CenterPane()
            self.oApi.PaneBorder(False)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
            return wNb
            oWid=ldmWidSizeRspWid(wNb, -1,mgr=self.mgrMain)
            wNb.AddPage(oWid,'first',True)
        except:
            self.logTB()
        return None
    def __initPanLf__(self,**kwargs):
        """initialize pane left
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initPanLf__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            oWid=ldmWidTree(parent=self.GetWid(),iLv=0,size=(200,120),sLogger='tr')
            self.oApi.Name('trDat')
            self.oApi.Caption("tree 0")
            self.oApi.Dockable(True)
            self.mgrMain.AddPane(oWid.GetWid(),self.oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initPanBt__(self,**kwargs):
        """initialize pane button
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initPanBt__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            oWid=ldmWidList(parent=self.GetWid(),iLv=0,
                        size=(200,120),sLogger='lstLog',
                        lCol=[
                            ['No',      'lf',80],
                            ['Info',    'lf',250],
                            ['Status',  'rg',60],
                        ])
            self.oApi.Name('lstLog')
            self.oApi.Caption("logging")
            self.mgrMain.AddPane(oWid.GetWid(),self.oApi)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initMenuBar__(self,**kwargs):
        """initialize menu bar
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initMenuBar__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            mnBar = wx.MenuBar()

            mnFile=wx.Menu()
            mnFile.Append(self.ID_FILE_OPEN, "&Open\tCtrl+O")
            mnFile.Append(self.ID_FILE_SAVE, "&Save\tCtrl+S")
            w=mnFile.Append(wx.ID_EXIT, "Exit\tAlt+X")
            self.BindEvent('mn',self.OnExit,w)
            
            mnEdit=wx.Menu()
            mnEdit.Append(self.ID_EDIT_CUT, "Cut\tCtrl+X")
            mnEdit.Append(self.ID_EDIT_COPY, "Copy\tCtrl+C")
            mnEdit.Append(self.ID_EDIT_PASTE, "Paste\tCtrl+V")

            iEn=self.getCfgWid('menubar','command',sType='int',oDft=0)
            if iEn>0:
                mnCmd=wx.Menu()
                iEn=self.getCfgWid('mnrCmd','00.enable',sType='int',oDft=1)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','00.menu',oDft='command 0')
                    w=mnCmd.Append(self.ID_CMD+0, sLbl)
                    self.BindEvent('mn',self.OnCmd00,w)
                
                iEn=self.getCfgWid('mnrCmd','01.enable',sType='int',oDft=1)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','01.menu',oDft='command 1')
                    w=mnCmd.Append(self.ID_CMD+1, sLbl)
                    self.BindEvent('mn',self.OnCmd01,w)
                
                iEn=self.getCfgWid('mnrCmd','02.enable',sType='int',oDft=1)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','02.menu',oDft='command 2')
                    w=mnCmd.Append(self.ID_CMD+2, sLbl)
                    self.BindEvent('mn',self.OnCmd02,w)
                
                iEn=self.getCfgWid('mnrCmd','03.enable',sType='int',oDft=1)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','03.menu',oDft='command 3')
                    w=mnCmd.Append(self.ID_CMD+3, sLbl)
                    self.BindEvent('mn',self.OnCmd03,w)
                
                iEn=self.getCfgWid('mnrCmd','04.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','04.menu',oDft='command 4')
                    w=mnCmd.Append(self.ID_CMD+4, sLbl)
                    self.BindEvent('mn',self.OnCmd04,w)
                
                iEn=self.getCfgWid('mnrCmd','05.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','05.menu',oDft='command 5')
                    w=mnCmd.Append(self.ID_CMD+5, sLbl)
                    self.BindEvent('mn',self.OnCmd05,w)
                
                iEn=self.getCfgWid('mnrCmd','06.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','06.menu',oDft='command 6')
                    w=mnCmd.Append(self.ID_CMD+6, sLbl)
                    self.BindEvent('mn',self.OnCmd06,w)
                
                iEn=self.getCfgWid('mnrCmd','07.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','07.menu',oDft='command 7')
                    w=mnCmd.Append(self.ID_CMD+7, sLbl)
                    self.BindEvent('mn',self.OnCmd07,w)
                
                iEn=self.getCfgWid('mnrCmd','08.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','08.menu',oDft='command 8')
                    w=mnCmd.Append(self.ID_CMD+8, sLbl)
                    self.BindEvent('mn',self.OnCmd08,w)
                
                iEn=self.getCfgWid('mnrCmd','09.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrCmd','09.menu',oDft='command 9')
                    w=mnCmd.Append(self.ID_CMD+9, sLbl)
                    self.BindEvent('mn',self.OnCmd09,w)
            else:
                mnCmd=None
            
            iEn=self.getCfgWid('menubar','destination',sType='int',oDft=0)
            if iEn>0:
                mnDst=wx.Menu()
                iEn=self.getCfgWid('mnrDst','00.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','00.menu',oDft='dest 0')
                    w=mnDst.Append(self.ID_DST+0, sLbl)
                    self.BindEvent('mn',self.OnDst00,w)
                
                iEn=self.getCfgWid('mnrDst','01.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','01.menu',oDft='dest 1')
                    w=mnDst.Append(self.ID_DST+1, sLbl)
                    self.BindEvent('mn',self.OnDst01,w)
                
                iEn=self.getCfgWid('mnrDst','02.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','02.menu',oDft='dest 2')
                    w=mnDst.Append(self.ID_DST+2, sLbl)
                    self.BindEvent('mn',self.OnDst02,w)
                
                iEn=self.getCfgWid('mnrDst','03.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','03.menu',oDft='dest 3')
                    w=mnDst.Append(self.ID_DST+3, sLbl)
                    self.BindEvent('mn',self.OnDst03,w)
                
                iEn=self.getCfgWid('mnrDst','04.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','04.menu',oDft='dest 4')
                    w=mnDst.Append(self.ID_DST+4, sLbl)
                    self.BindEvent('mn',self.OnDst04,w)
                
                iEn=self.getCfgWid('mnrDst','05.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','05.menu',oDft='dest 5')
                    w=mnDst.Append(self.ID_DST+5, sLbl)
                    self.BindEvent('mn',self.OnDst05,w)
                
                iEn=self.getCfgWid('mnrDst','06.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','06.menu',oDft='dest 6')
                    w=mnDst.Append(self.ID_DST+6, sLbl)
                    self.BindEvent('mn',self.OnDst06,w)
                
                iEn=self.getCfgWid('mnrDst','07.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','07.menu',oDft='dest 7')
                    w=mnDst.Append(self.ID_DST+7, sLbl)
                    self.BindEvent('mn',self.OnDst07,w)
                
                iEn=self.getCfgWid('mnrDst','08.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','08.menu',oDft='dest 8')
                    w=mnDst.Append(self.ID_DST+8, sLbl)
                    self.BindEvent('mn',self.OnDst08,w)
                
                iEn=self.getCfgWid('mnrDst','09.enable',sType='int',oDft=0)
                if iEn>0:
                    sLbl=self.getCfgWid('mnrDst','09.menu',oDft='dest 9')
                    w=mnDst.Append(self.ID_DST+9, sLbl)
                    self.BindEvent('mn',self.OnDst09,w)
            else:
                mnDst=None
            mnHelp=wx.Menu()
            mnHelp.Append(wx.ID_ABOUT, "About...")

            mnBar.Append(mnFile, "&File")
            mnBar.Append(mnEdit, "&Edit")
            if mnCmd is not None:
                sLbl=self.getCfgWid('mnrCmd','menu',oDft='Command')
                mnBar.Append(mnCmd, sLbl)
            if mnDst is not None:
                sLbl=self.getCfgWid('mnrDst','menu',oDft='Destination')
                mnBar.Append(mnDst, sLbl)
            mnBar.Append(mnHelp, "&Help")

            self.wid.SetMenuBar(mnBar)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initStatusBar__(self,**kwargs):
        """initialize status bar
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initStatusBar__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            self.wid.CreateStatusBar()
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initEvt__(self,**kwargs):
        """initialize event
        ### parameter
            kwargs  ... keyword arguments
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initEvt__'
            self.logDbg('beg:%s'%(sOrg))
            # ----- end:initialize
            # +++++ beg:
            #self.Bind(aui.EVT_AUITOOLBAR_TOOL_DROPDOWN, self.OnDropDownToolbarItem, id=ID_DropDownToolbarItem)
            #self.Bind(aui.EVT_AUI_PANE_CLOSE, self.OnPaneClose)
            #self.Bind(aui.EVT_AUINOTEBOOK_ALLOW_DND, self.OnAllowNotebookDnD)
            #self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.OnNotebookPageClose)

            #self.Bind(aui.EVT_AUI_PANE_FLOATING, self.OnFloatDock)
            #self.Bind(aui.EVT_AUI_PANE_FLOATED, self.OnFloatDock)
            #self.Bind(aui.EVT_AUI_PANE_DOCKING, self.OnFloatDock)
            #self.Bind(aui.EVT_AUI_PANE_DOCKED, self.OnFloatDock)
            
            self.wid.Bind(wx.EVT_ERASE_BACKGROUND, self.OnFrmEraseBackground)
            self.wid.Bind(wx.EVT_SIZE, self.OnFrmSize)
            self.wid.Bind(wx.EVT_CLOSE, self.OnFrmClose)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def GetDockArt(self):
        return self.mgrMain.GetArtProvider()
    def OnFileOpen(self,evt):
        """event handler
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnFileOpen')
        except:
            self.logTB()
    def OnFileSave(self,evt):
        """event handler
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnFileSave')
        except:
            self.logTB()
    def OnEditCut(self,evt):
        """event handler
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnEditCut')
        except:
            self.logTB()
    def OnEditCopy(self,evt):
        """event handler
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnEditCopy')
        except:
            self.logTB()
    def OnEditPaste(self,evt):
        """event handler of edit past
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnEditPaste')
        except:
            self.logTB()
    def prcCmd(self,sCmd):
        """process command, called by event default handler
        ### parameter
            sCmd    ... command name
                cmd00   ... command button 00, tbrCmd.00
                cmd01   ... command button 00, tbrCmd.01
                ..
                cmd09   ... command button 00, tbrCmd.09
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::prcCmd'
            self.logDbg('beg:%s sCmd:%s',sOrg,sCmd)
            iRet=0
            # ----- end:initialize
            # +++++ beg:
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1
    def OnCmd00(self,evt):
        """event handler for command button 00
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd00'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd00')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd01(self,evt):
        """event handler for command button 01
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd01'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd01')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd02(self,evt):
        """event handler for command button 02
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd02'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd02')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd03(self,evt):
        """event handler for command button 03
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd03'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd03')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd04(self,evt):
        """event handler for command button 04
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd04'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd04')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd05(self,evt):
        """event handler for command button 05
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd05'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd05')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd06(self,evt):
        """event handler for command button 06
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd06'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd06')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd07(self,evt):
        """event handler for command button 07
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd07'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd07')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd08(self,evt):
        """event handler for command button 08
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd08'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd08')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd09(self,evt):
        """event handler for command button 09
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnCmd09'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('cmd09')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst00(self,evt):
        """event handler for destination button 00
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst00'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst00')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst01(self,evt):
        """event handler for destination button 01
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst01'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst01')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst02(self,evt):
        """event handler for destination button 02
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst02'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst02')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst03(self,evt):
        """event handler for destination button 03
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst03'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst03')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst04(self,evt):
        """event handler for destination button 04
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst04'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst04')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst05(self,evt):
        """event handler for destination button 05
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst05'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst05')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst06(self,evt):
        """event handler for destination button 06
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst06'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst06')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst07(self,evt):
        """event handler for destination button 07
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst07'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst07')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst08(self,evt):
        """event handler for destination button 08
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst08'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst08')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnDst09(self,evt):
        """event handler for destination button 09
        ### parameter
            evt     ... event object
        """
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::OnDst09'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            iRet=self.prcCmd('dst09')
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnFrmEraseBackground(self, event):
        """event handler
        ### parameter
            evt     ... event object
        """
        event.Skip()
    def OnFrmSize(self, event):
        """event handler
        ### parameter
            evt     ... event object
        """
        event.Skip()
    def OnFrmClose(self, event):
        """event handler
        ### parameter
            evt     ... event object
        """
        #self.timer.Stop()
        self.mgrMain.UnInit()
        event.Skip()
    def OnExit(self, event):
        """event handler
        ### parameter
            evt     ... event object
        """
        try:
            self.logDbg('OnExit')
            self.wid.Close(True)
        except:
            self.logTB()
    def DoUpdate(self):
        """event handler
        ### parameter
            evt     ... event object
        """
        self.mgrMain.Update()
        self.Refresh()



#----------------------------------------------------------------------------
# Name:         lmdGuiThread.py
# Purpose:      gui thread class
#
# Author:       Walter Obweger
#
# Created:      20200404
# CVS-ID:       $Id$
# Copyright:    Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import time
import wx
import wx.lib.newevent

from six.moves import _thread as sixThd

from lindworm.logUtil import ldmUtilLog
from lindworm.ldmGuiNty import ldmGuiNty

# +++++ beg:create event class and binder function
(ldmGuiNotifyEvt, EVT_LDM_GUI_NOTIFY) = wx.lib.newevent.NewEvent()
# ----- end:initialize

class ldmGuiThd(ldmUtilLog):
    def __init__(self,oGui,rDly=0.200,sLogger='ldmGuiThd',iLv=0,iVerbose=0):
        """constructor
        ### parameter
            oGui        ... GUI object
            rDly        ... delay between processing [s], yield GIL
            sLogger     ... log origin
            iLv         ... log level short
            iVerbose    ... higher values add more logs
        """
        ldmUtilLog.__init__(self,sLogger=sLogger,iLv=iLv,
                            iVerbose=iVerbose)
        self.oGui=oGui
        self.iKeepRun=0
        self.iRunning=0
        self.rDly=max(rDly,0.01)
        self.lCall=[]
        self.oNty=ldmGuiNty()
    def BindEvtNty(self,funcOnNty):
        """bind event function to oGui object
        ### parameter
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            if self.oGui is not None:
                self.oGui.Bind(EVT_LDM_GUI_NOTIFY, funcOnNty)
                return 1
            else:
                return 0
        except:
            self.logTB()
            return -1
    def GetNty(self):
        """get notify object
        ### parameter
        ### return
            !=None  ... okay processing done
            ==None  ... okay nop
        """
        return self.oNty
    def Do(self,sPhase,func,*args,**kwargs):
        """schedule callable
        ### parameter
            sPhase  ... phase name
            fun     ... callable
            args    ... argument
            kwargs  ... keyword arguments
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            # ----- end:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmGuiThd::'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            # ----- end:initialize
            # +++++ beg:
            self.oNty.IncSchedule()
            self.lCall.append((sPhase,func,args,kwargs))
            # ----- end:
            # +++++ beg:
            if self.iKeepRun<1:
                self.Start()
                iRet=2
            else:
                iRet=1
            # ----- end:
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1
    def Start(self):
        self.iKeepRun=1
        self.iRunning=1
        self.iRunExc=1
        sixThd.start_new_thread(self._runNotify,())
        sixThd.start_new_thread(self._runExc,())
    def Stop(self):
        self.iKeepRun=0
    def _runNotify(self):
        """
        ### parameter
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            # ----- end:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmGuiThd::_runNotify'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            # ----- end:initialize
            # +++++ beg:
            while (self.iKeepRun>0) and (self.iRunExc>0):
                if self.oGui is not None:
                    oNtyDat=self.oNty.GetNtyDat()
                    if oNtyDat is not None:
                        evt = ldmGuiNotifyEvt(oNty = oNtyDat)
                        wx.PostEvent(self.oGui, evt)
                    #oNtyDat=ldmGuiNtyDat(oNty=self.oNty)
                    #evt = ldmGuiNotifyEvt(oNty = oNtyDat)
                    #wx.PostEvent(self.oGui, evt)
                iAct=self.oNty.IsActive()
                if self.iVerbose>0:
                    self.oLog.debug('    %s iAct:%d sleep:%4.3f'%(sOrg,
                                    iAct,self.rDly))
                time.sleep(self.rDly)
            # ----- end:
            # +++++ beg:
            iAct=self.oNty.IsActive()
            if self.oGui is not None:
                oNtyDat=self.oNty.GetNtyDat()
                if oNtyDat is not None:
                    evt = ldmGuiNotifyEvt(oNty = oNtyDat)
                    wx.PostEvent(self.oGui, evt)
                #self.oNty.SetStatus('stopped')
            #    evt = ldmGuiNotifyEvt(oNty = self.oNty)
            #    wx.PostEvent(self.oGui, evt)
            #    time.sleep(self.rDly)
            self.iRunning=0
            # ----- end:
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1
    def delay(self,iCnt=1):
        time.sleep(self.rDly)
        while iCnt>0:
            time.sleep(self.rDly)
            iCnt-=1
    def _runExc(self):
        """
        ### parameter
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            # ----- end:
            # +++++ beg:initialize
            iRet=0
            sOrg='ldmGuiThd::_runExc'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            # ----- end:initialize
            # +++++ beg:
            while self.iKeepRun>0:
                if len(self.lCall)>0:
                    try:
                        sPhase,func,args,kwargs=self.lCall.pop(0)
                        self.oNty.SetPhase(sPhase)
                        oRetCall=func(*args,**kwargs)
                        self.oLog.debug('    %s oRetCall:%r'%(sOrg,oRetCall))
                    except:
                        self.logTB()
                if self.iVerbose>0:
                    self.oLog.debug('    %s sleep:%4.3f'%(sOrg,self.rDly))
                time.sleep(self.rDly)
            # +++++ beg:
            self.iRunExc=0
            # ----- end:
            # ----- end:
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1
    def __tpl(self):
        """
        ### parameter
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            # ----- end:
            # +++++ beg:initialize
            iRet=0
            sOrg='::'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            # ----- end:initialize
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# generated by wxGlade 0.9.5 on Thu Apr  2 08:19:01 2020
#
#----------------------------------------------------------------------------
# Name:         ldmStorageFolderViewFrm.py
# Purpose:      ldmStorageFolderViewFrm.py
#               GUI viewer for ldmStorageFolder
# Author:       Walter Obweger
#
# Created:      20200404
# CVS-ID:       $Id$
# Copyright:    (c) 2020 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------


import wx
import wx.lib.gizmos as gizmos
import os
import logging
import traceback

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade

from lindworm import __version__
import lindworm.ldmGui as ldmGui
from lindworm.ldmArg import ldmArg
from lindworm.ldmWidApp import ldmWidApp
from lindworm.ldmWidFrmAui import ldmWidFrmAui
from lindworm.ldmStorageFolder import ldmStorageFolder

gOpt=None

class ldmStorageFolderViewFrm(ldmWidFrmAui):
    WILDCARD_JSON=["json file (*.json)|*.json",
            "All files (*.*)|*.*"]
    def __initPanLf__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderViewFrm::__initPanLf__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def __initPanCt__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderViewFrm::__initPanLf__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            wNb=ldmWidFrmAui.__initPanCt__(self,**kwargs)
            self.widMainNB=wNb
            self.trlFdr = gizmos.TreeListCtrl(self.GetWid(), 
                                    wx.ID_ANY, 
                                    size=(400,300),
                                    style=0, 
                                    agwStyle=gizmos.TR_DEFAULT_STYLE|
                                    wx.TR_HIDE_ROOT|
                                    #| gizmos.TR_HAS_BUTTONS
                                    #| gizmos.TR_TWIST_BUTTONS
                                    #| gizmos.TR_ROW_LINES
                                    #| gizmos.TR_COLUMN_LINES
                                    #| gizmos.TR_NO_LINES
                                    #| gizmos.TR_LINES_AT_ROOT
                                    gizmos.TR_FULL_ROW_HIGHLIGHT
                                    )
            tSz = (16,16)
            il = wx.ImageList(*tSz)
            iIdxFdrCls  = il.Add(wx.ArtProvider.GetIcon(wx.ART_FOLDER,      wx.ART_OTHER, tSz))
            iIdxFdrOpn  = il.Add(wx.ArtProvider.GetIcon(wx.ART_FOLDER_OPEN, wx.ART_OTHER, tSz))
            iIdxFile    = il.Add(wx.ArtProvider.GetIcon(wx.ART_NORMAL_FILE, wx.ART_OTHER, tSz))
            self.dImgIdx={
                'FdrCls':iIdxFdrCls,
                'FdrOpn':iIdxFdrOpn,
                'File':iIdxFile,
                }
            self.trlFdr.SetImageList(il)
            self.trlFdr.AddColumn('name')
            self.trlFdr.AddColumn('size')
            self.trlFdr.AddColumn('time')
            self.trlFdr.AddColumn('sha')
            self.trlFdr.SetMainColumn(0)
            self.trlFdr.SetColumnWidth(0, 250)
            wNb.AddPage(self.trlFdr,'folder content',True)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
            return wNb
        except:
            self.logTB()
            return None
    def __initCfg__(self):
        try:
            # +++++ beg:initialize
            sOrg='ldmWidFrmAui::__initCfg__'
            self.logDbg('beg:%s'%(sOrg))
            ldmWidFrmAui.__initCfg__(self,**kwargs)
            # ----- end:initialize
            # +++++ beg:
            self.CHC_DEF=["slot 00","slot 01","slot 02","slot 03"]
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initDat__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderViewFrm::__initDat__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            #self.sCfgFN=gOpt.sCfgFN      #"ldmStorageFolderCfg.json"
            #self.sBldFN=gOpt.sBldFN             #None
            #self.iVerbose=10
            # +++++ beg:
            self.oFdr=ldmStorageFolder(iVerbose=self.iVerbose)
            #iRet=oFdr.loadCfg(sCfgFN)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def __initPrp__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderViewFrm::__initPrp__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            sBldFN=self.pnCLI.txtBldFN.GetValue()
            self.logDbg('sBldFN:%s',sBldFN)
            iRet=self.shwFdr(sBldFN,sKey='0')
            if iRet>0:
                self.widMainNB.SetSelection(0)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()

    def shwFdr(self,sFN,sKey='0'):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderViewFrm::shwFdr'
            self.logDbg('beg:%s sFN:%s sKey:%r',
                        sOrg,sFN,sKey)
            iRet=0
            self.trlFdr.DeleteAllItems()
            # ----- end:initialize
            # +++++ beg:
            iRet=self.oFdr.loadDat(sFN,sDN=None,
                            sKey=sKey,sAtr='dDat')
            itPar=self.trlFdr.AddRoot(sKey)
            if iRet>0:
                #dFdr=self.oFdr.dDat[sKey]
                iR,dFdr=self.oFdr.getDat(sKey)
                def getFmtSzHum(iSz):
                    fSz=iSz/1024/1024
                    return '%6.3f'%fSz
                def addElemFle(trlObj,itPar,dFdr):
                    iRet=0
                    iIdxFleOpn=self.dImgIdx['File']
                    iIdxFleCls=self.dImgIdx['File']
                    for sFdr,dSub in dFdr.items():
                        iRet+=1
                        itCur=trlObj.AppendItem(itPar, sFdr)
                        trlObj.SetItemImage(itCur, iIdxFleOpn, which = wx.TreeItemIcon_Normal)
                        #trlObj.SetItemImage(itCur, iIdxFdrCls , which = wx.TreeItemIcon_Expanded)
                        sSz=getFmtSzHum(dSub.get('sz',0))
                        trlObj.SetItemText(itCur,sSz, 1)
                        sTime=dSub.get('time','')
                        trlObj.SetItemText(itCur,sTime, 2)
                        sSha=dSub.get('sha','')
                        trlObj.SetItemText(itCur,sSha, 3)
                    return iRet
                def addElemFdr(trlObj,itPar,dFdr):
                    iRet=0
                    iIdxFdrOpn=self.dImgIdx['FdrOpn']
                    iIdxFdrCls=self.dImgIdx['FdrCls']
                    for sFdr,dSub in dFdr.items():
                        if sFdr=='.':
                            iRet+=1
                            itCur=trlObj.AppendItem(itPar, sFdr)
                            trlObj.SetItemImage(itCur, iIdxFdrOpn, which = wx.TreeItemIcon_Normal)
                            trlObj.SetItemImage(itCur, iIdxFdrCls , which = wx.TreeItemIcon_Expanded)
                            iR=addElemFle(trlObj,itCur,dSub)
                            if iR>0:
                                iRet+=1
                        else:
                            iRet+=1
                            itCur=trlObj.AppendItem(itPar, sFdr)
                            trlObj.SetItemImage(itCur, iIdxFdrOpn, which = wx.TreeItemIcon_Normal)
                            trlObj.SetItemImage(itCur, iIdxFdrCls , which = wx.TreeItemIcon_Expanded)
                            iR=addElemFdr(trlObj,itCur,dSub)
                            if iR>0:
                                iRet+=1
                    return iRet
                iR=addElemFdr(self.trlFdr,itPar,dFdr)
            # ----- end:initialize
            # +++++ beg:get DN
            self.logDbg('end:shwFdr iRet:%d sFN:%s',
                                iRet,sFN)
            # ----- end:get DN
            return iRet
        except:
            self.oFdr.logTB()
    def OnFileOpen(self,evt):
        try:
            self.logDbg('beg:OnFileOpen')
            # +++++ beg:get DN
            sBldFN='.'#self.txtBldFN.GetValue()
            # ----- end:get DN
            iRet,sBldFN=ldmGui.getFN(sBldFN,self.GetWid(),
                         'choose build file name',
                         lWildCard=self.WILDCARD_JSON,
                         oLog=self.oFdr)
            if iRet>0:
                iR=self.shwFdr(sBldFN,sKey='0')
                if iR>0:
                    iRet+=iR
                self.oFdr.log('fin:OnBldFN iRet:%d sBldFN:%s',
                                iRet,sBldFN)
            self.logDbg('end:OnFileOpen')
        except:
            self.logTB()
    def OnCfgFN(self, event):  # wxGlade: ldmStorageFolderViewFrm.<event_handler>
        event.Skip()
        try:
            sCfgFN=self.txtCfgFN.GetValue()
            iRet,sCfgFN=ldmGui.getFN(sCfgFN,self,
                         'choose configuration file name',
                         lWildCard=self.WILDCARD_JSON,
                         oLog=self.oFdr)
            if iRet>0:
                self.sCfgFN=sCfgFN
                self.txtCfgFN.SetValue(self.sCfgFN)
                self.oFdr.log('fin:OnCfgFN iRet:%d sCfgFN:%s',
                            iRet,sCfgFN)
        except:
            self.oFdr.logTB()

def main(args=None):
    # +++++ beg:
    # ----- end:
    
    # +++++ beg:init
    iRet=0
    iVerbose=5                                          # 20190624 wro:set default verbose level
    # ----- end:init
    # +++++ beg:define CLI arguments
    usage = "usage: %prog [options]"
    oArg=ldmArg(sUsage=usage,sVer=__version__,iVerbose=0)
    oArg.addOpt('sCfgFN',
            sDft='ldmStorageFolderViewCfg.json',
            sHlp='configuration file',
            sVerbose='config FN',
            sMeta='pyGatherMDCfg.json')
    oArg.addOpt('sBldFN',
            sDft='',
            sHlp='build directory',
            sVerbose='build FN',
            sMeta='path/to/output/folder')
    oArg.addOpt('sLogFN',
            sDft='./log/ldmStorageFolder.log',
            sHlp='log filename',
            sVerbose='log FN',
            sMeta='./log/ldmStorageFolder.log')

    # ----- end:define CLI arguments
    # +++++ beg:parse command line
    iRet=oArg.prcParse(args)
    # ----- end:parse command line
    # +++++ beg:prepare logging
    if oArg.sLogFN is not None:
        import lindworm.logUtil as logUtil
        logUtil.logInit(oArg.sLogFN,iLevel=logging.DEBUG)
    # ----- end:prepare logging
    # +++++ beg:
    app = ldmWidApp(0,
                title="lindworm storage folder viewer",
                oArg=oArg,
                clsFrm=ldmStorageFolderViewFrm)
    app.MainLoop()
    iRet+=1
    # ----- end:
    return iRet

if __name__ == "__main__":
    # +++++ beg:call entry point
    main(args=None)
    # ----- end:call entry point



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# generated by wxGlade 0.9.5 on Thu Apr  2 08:19:01 2020
#
#----------------------------------------------------------------------------
# Name:         ldmStorageFolderFrm.py
# Purpose:      ldmStorageFolderFrm.py
#               GUI for ldmStorageFolder
# Author:       Walter Obweger
#
# Created:      20200402
# CVS-ID:       $Id$
# Copyright:    (c) 2020 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------


import wx
import os
import logging
import traceback

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade

from lindworm import __version__
import lindworm.ldmGui as ldmGui
from lindworm.ldmArg import ldmArg
from lindworm.ldmWidApp import ldmWidApp
from lindworm.ldmWidFrmAui import ldmWidFrmAui
from lindworm.ldmGuiThd import ldmGuiThd
from lindworm.ldmWidTree import ldmWidTree
from lindworm.ldmWidThd import ldmWidThd
from lindworm.ldmStorageFolder import ldmStorageFolder

class ldmStorageFolderThd(ldmWidThd):
    def prcCmd(self,sCmd):
        try:
            # +++++ beg:
            self.logDbg('ldmStorageFolderFrm::prcCmd sCmd:%r',sCmd)
            if sCmd=='start':
                oNty=self.oThd.GetNty()
                self.oThd.Do('%s'%(self.sSrcDN),
                            self.prcFld,*(),
                            sCfgFN=self.sCfgFN,
                            sSrcDN=self.sSrcDN,
                            sBldFN=self.sBldFN,
                            sBldDN=self.sBldDN,
                            iShaMB=self.iShaMB,
                            oNty=oNty)
            # ----- end:
            return 0
        except:
            self.logTB()
            return -1

class ldmStorageFolderFrm(ldmWidFrmAui):
    WILDCARD_JSON=["json file (*.json)|*.json",
            "All files (*.*)|*.*"]
    def __initCfg__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::__initCfg__'
            self.logDbg('beg:%s'%(sOrg))
            ldmWidFrmAui.__initCfg__(self,**kwargs)
            # ----- end:initialize
            # +++++ beg:
            self.setCfgWid('toolbar','command','1')
            #self.dCfgWidDft['frmMain']['sToolBarCommand']='1'
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initPanLf__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::__initPanLf__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def __initPanCt__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::__initPanLf__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            wNb=ldmWidFrmAui.__initPanCt__(self,**kwargs)
            self.widMainNB=wNb
            # ----- end:
            # +++++ beg:
            oWid=ldmWidThd(parent=wNb,iLv=0,
                        size=(200,120),sLogger='thdFdr',
                        )
            wNb.AddPage(oWid.GetWid(),'thd')
            self.wThd=oWid
            # ----- end:
            self.wThd.SetFunc('start','launch',self.DoStart)
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
            return wNb
        except:
            self.logTB()
            return None
    def OnCmd00(self,evt):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::OnCmd00'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def OnCmd01(self,evt):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::OnCmd00'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            oThd=self.wThd.GetThd()
            self.DoStart(oThd)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s'%(sOrg))
            # ----- end:finalize
        except:
            self.logTB()
    def __initDat__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::__initDat__'
            self.logDbg('beg:%s',sOrg)
            # ----- end:initialize
            # +++++ beg:
            self.iShaMB=1
            # ----- end:
            # +++++ beg:
            self.oFld=ldmStorageFolder(iLv=0,iVerbose=self.iVerbose)
            #iRet=oFld.loadCfg(sCfgFN)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def __initPrp__(self,**kwargs):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::__initPrp__'
            self.logDbg('beg:%s',sOrg)
            iRet=0
            # ----- end:initialize
            # +++++ beg:
            sBldFN=self.pnCLI.txtBldFN.GetValue()
            self.logDbg('sBldFN:%s',sBldFN)
            #iRet=self.shwFdr(sBldFN,sKey='0')
            if iRet>0:
                self.widMainNB.SetSelection(0)
            # ----- end:
            # +++++ beg:finalize
            self.logDbg('end:%s',sOrg)
            # ----- end:finalize
        except:
            self.logTB()
    def DoStart(self,oThd=None):
        try:
            # +++++ beg:initialize
            logging.debug('beg:OnStart')
            iRet=0
            # ----- end:initialize
            # +++++ beg:
            #sSrcDN=self.txtSrcDN.GetValue()
            sSrcDN=self.pnCLI.txtSrcDN.GetValue()
            sCfgFN=self.pnCLI.txtCfgFN.GetValue()
            sBldFN=self.pnCLI.txtBldFN.GetValue()
            sBldDN=self.pnCLI.txtBldDN.GetValue()
            #sSrcDN=self.pnCLI.txtSrcDN.GetValue()
            iShaMB=self.pnCLI.spnShaMbInt.GetValue()
            #iShaMB=self.GetShaMbChoice()
            # ----- end:
            # +++++ beg:schedule action
            oNty=oThd.GetNty()
            oThd.Do('%s'%(sSrcDN),
                        self.prcFld,*(),
                        sCfgFN=sCfgFN,
                        sSrcDN=sSrcDN,
                        sBldFN=sBldFN,
                        sBldDN=sBldDN,
                        iShaMB=iShaMB,
                        oNty=oNty)
            self.widMainNB.SetSelection(0)
            # ----- end:schedule action
            # +++++ beg:finalize
            logging.debug('end:OnStart iRet:%d'%(iRet))
            # ----- end:finalize
            return 1
        except:
            self.oFld.logTB()
    def prcFld(self,sCfgFN=None,sSrcDN=None,
                sBldFN=None,sBldDN=None,
                iShaMB=0,oNty=None):
        try:
            # +++++ beg:initialize
            sOrg='ldmStorageFolderFrm::prcFld'
            self.oFld.logDbg('beg:%s sSrcDN:%s',sOrg,sSrcDN)
            self.oFld.clrAll()
            iRet=self.oFld.loadCfg(sCfgFN)
            # ----- end:initialize
            # +++++ beg:register action
            #self.oFld.logDbg('    sSrcDN:%r',sSrcDN)
            iRet=self.oFld.prcBeg(sSrcDN,oNty=oNty,oRef=None)
            if iRet>0:
                if oNty is not None:
                    oNty.SetPhase('%r ...'%(sSrcDN))
                    oNty.SetStatus('')
                iRet+=1
            else:
                if oNty is not None:
                    oNty.SetPhase('%r skipped. iRet:%d'%(sSrcDN,iRet))
                    oNty.SetStatus('failed start')
                return -1
            # ----- end:register action
            # +++++ beg:perform action
            if oNty is not None:
                oNty.SetPhase('%r prepare ...'%(sSrcDN))
                oNty.SetStatus('')
            iRet=self.oFld.prcExc(oNty=oNty)#(oGtrMD=self)
            if iRet>0:
                if oNty is not None:
                    oNty.SetPhase('%r finalize ...'%(sSrcDN))
            else:
                if oNty is not None:
                    oNty.SetPhase('%r aborted. iRet:%d'%(sSrcDN,iRet))
                    oNty.SetStatus('failed to process')
                return -2
            # ----- end:perform action
            # +++++ beg:finalize action
            self.oFld.logDbg('    %s iShaMB:%r',sOrg,iShaMB)
            iRetFdr=self.oFld.prcEnd(iShaMB=iShaMB,oNty=oNty)
            if iRetFdr>0:
                if oNty is not None:
                    oNty.SetPhase('%r finished.'%(sSrcDN))
                iRet+=1
            else:
                if oNty is not None:
                    oNty.SetPhase('%r aborted. iRet:%d'%(sSrcDN,iRetFdr))
                    oNty.SetStatus('failed to finalize')
            # ----- end:finalize action
            # +++++ beg:save result to json
            if iRetFdr>0:
                self.oFld.logDbg('    %s sBldFN:%r sBldDN:%r sDefEnd:%r',
                            sOrg,sBldFN,sBldDN,self.oFld.sDefEnd)
                if sBldFN is None:
                    sBldFN=None
                else:
                    if len(sBldFN)>0:
                        if sBldFN=='0':
                            sBldFN=0
                        elif sBldFN=='1':
                            sBldFN=1
                    else:
                        sBldFN=None
                iRetSave=self.oFld.saveDat(sBldFN,sDN=sBldDN,
                                    lKey=[self.oFld.sDefEnd],
                                    sAtr='dDat')
                if iRetSave>0:
                    if oNty is not None:
                        oNty.SetStatus('saved FN:%r DN:%r'%(sBldFN,sBldDN))
                    iRet+=1
                else:
                    oNty.SetStatus('save failed iRetSave:%d'%(iRetSave))
            # ----- end:save result to json
            # +++++ beg:
            if oNty is not None:
                oNty.finStatus()
                #self.oThd.delay(iCnt=5)
            self.oFld.logDbg('end:%s iRet:%d',sOrg,iRet)
            # ----- end:
            return iRet
        except:
            self.oFld.logTB()
            return -1


# end of class ldmStorageFolderFrm


def main(args=None):
    # +++++ beg:
    # ----- end:
    
    # +++++ beg:init
    iRet=0
    iVerbose=5                                          # 20190624 wro:set default verbose level
    # ----- end:init
    # +++++ beg:define CLI arguments
    usage = "usage: %prog [options]"
    oArg=ldmArg(sUsage=usage,sVer=__version__,iVerbose=0)
    oArg.addOpt('sCfgFN',
            sDft='ldmStorageFolderCfg.json',
            sHlp='configuration file',
            sVerbose='config FN',
            sMeta='pyGatherMDCfg.json')
    oArg.addOpt('sSrcDN',
            sDft='./',
            sHlp='source folder',
            sVerbose='source DN',
            sMeta='path/to/folder/to/read')
    oArg.addOpt('sBldDN',
            sDft='./',
            sHlp='build directory',
            sVerbose='build DN',
            sMeta='path/to/output/folder')
    oArg.addOpt('sBldFN',
            sDft='./',
            sHlp='build directory',
            sVerbose='build FN',
            sMeta='path/to/output/folder')
    oArg.addOpt('sShaMbInt',
            sDft='1',
            sHlp='size boundary;-1 <= x <= 128',
            sVerbose='limit',
            sMeta='full')
    oArg.addOpt('sLogFN',
            sDft='./log/ldmStorageFolder.log',
            sHlp='log filename',
            sVerbose='log FN',
            sMeta='./log/ldmStorageFolder.log')
    # ----- end:define CLI arguments
    # +++++ beg:parse command line
    iRet=oArg.prcParse(args)
    if oArg.GetVerbose(10):
        print('ldmArg.prcParse:%d'%(iRet))
        print(" log FN:     %r"%(oArg.sLogFN))
    # ----- end:parse command line
    # +++++ beg:prepare logging
    if oArg.sLogFN is not None:
        import lindworm.logUtil as logUtil
        logUtil.logInit(oArg.sLogFN,iLevel=logging.DEBUG)
    # ----- end:prepare logging
    # +++++ beg:
    app = ldmWidApp(0,
                title="lindworm storage folder",
                oArg=oArg,
                clsFrm=ldmStorageFolderFrm)
    app.MainLoop()
    iRet+=1
    # ----- end:
    return iRet

if __name__ == "__main__":
    # +++++ beg:call entry point
    main(args=None)
    # ----- end:call entry point



#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         setup.py
# Purpose:      setup definition
#               
# Author:       Walter Obweger
#
# Created:      20191207
# CVS-ID:       $Id$
# Copyright:    (c) 2019 by Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import io
import os
import re
import sys
from setuptools import setup

# +++++ beg:get version number from package init
with open('lindworm/__init__.py', 'r') as oFile:
    sVersion = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        oFile.read(), re.MULTILINE).group(1)
# ----- end:get version number from package init

# +++++ beg:use readme as description
# Use the README.md content for the long description:
with io.open("lindworm/README.md", encoding="utf-8") as oFile:
    sDescLong = oFile.read()
# ----- end:use readme as description

setup(
    name='lindworm',
    version=sVersion,
    description='core tools covering folder content, markdown, GUI',
    long_description=sDescLong,
    long_description_content_type="text/markdown",
    author='Walter Obweger',
    author_email='walter.obweger@gmail.com',
    url='https://gitlab.com/wobweger/lindworm',
    project_urls={
        'Documentation': 'https://gitlab.com/wobweger/lindworm',
    },
    license='MIT',
    packages=[
        'lindworm',
        #'lindworm.pyGatherMD',
        #'lindworm.mt',
        #'lindworm.pcs7',
        #'lindworm.pd',
        #'lindworm.xd',
    ],
    test_suite='tests',
    install_requires=[
        'pandas',
        'pysimplegui',
        #'pyobjc-core;platform_system=="Darwin"', 
        #'pyobjc;platform_system=="Darwin"',
        #'python3-Xlib;platform_system=="Linux" and python_version>="3.0"', 'Xlib;platform_system=="Linux" and python_version<"3.0"',
        #'pymsgbox', 
        #'PyTweening>=1.0.1', 
        #'pyscreeze>=0.1.21', 
        #'pygetwindow>=0.0.5', 
        #'mouseinfo'
        ],
    
    keywords="tools",
    classifiers=[
        'Development Status :: 1 - Planning',
        #'Development Status :: 2 - Pre-Alpha',
        #'Development Status :: 3 - Alpha',
        #'Development Status :: 4 - Beta',
        #'Development Status :: 5 - Production/Stable',
        #'Development Status :: 6 - Mature',
        'Environment :: Console',
        'Environment :: Win32 (MS Windows)',
        'Environment :: X11 Applications :: GTK',
        #'Environment :: X11 Applications',
        #'Environment :: MacOS X',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        #'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        #'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        #'Programming Language :: Python :: Implementation :: CPython',
        #'Programming Language :: Python :: Implementation :: PyPy',
        #'Topic :: Desktop Environment :: File Managers',
        'Topic :: Documentation',
        #'Topic :: Office/Business',
        #'Topic :: Scientific/Engineering',
        #'Topic :: Software Development',
        #'Topic :: System',
        #'Topic :: System :: Archiving',
        #'Topic :: System :: Distributed Computing',
        #'Topic :: System :: Filesystems',
        #'Topic :: System :: Operating System',
        #'Topic :: Text Processing',
        'Topic :: Utilities',
    ],
    entry_points={
        'console_scripts': [
            'lindworm.pyGatherMD = lindworm.pyGatherMD:main',
            'lindworm.ldmStorageFolder = lindworm.ldmStorageFolder:main',
        ],
        'gui_scripts': [
            'lindworm.ldmStorageFolderFrm = lindworm.ldmStorageFolderFrm:main',
        ]
    },
)



#----------------------------------------------------------------------------
# Name:         pyGatherMD.py
# Purpose:      python gather markdown
#
# Author:       Walter Obweger
#
# Created:      20191102
# CVS-ID:       $Id$
# Copyright:    Walter Obweger
# Licence:      MIT
#----------------------------------------------------------------------------

import os
import os.path
import logging
import logging.handlers
import traceback
import shutil
import json

from optparse import OptionParser

from lindworm import __version__
from lindworm.ldmStorageLine import ldmStorageLine
from lindworm.ldmMD import ldmMD

class pyGatherMD(ldmStorageLine):
    def __init__(self,sLogger='srgMD',iLv=1,iVerbose=0):
        """constructor
        ### parameter
            iLv         ... logging level
            sLogger     ... log origin
            iLv         ... logging level
            iVerbose    ... higher values add more logs
        """
        self.sFN=None
        #self.oSrgMD=ldmStorageLine(sLogger='srgMD',iVerbose=iVerbose-10)
        ldmStorageLine.__init__(self,sLogger='srgMD',
                            iLv=1,
                            iVerbose=iVerbose-10)
        self.__initDat__()
    def __initCfg__(self):
        """initialize configuration properties
        """
        try:
            # +++++ beg:
            ldmStorageLine.__initCfg__(self)
            # ----- end:
            # +++++ beg:
            # ----- end:
        except:
            self.logTB()
            return -1
    def __initDat__(self):
        """initialize data properties
        """
        try:
            # +++++ beg:
            ldmStorageLine.__initDat__(self)
            # ----- end:
            # +++++ beg:
            self.sInFN=''
            self.sOtFN=''
            self.lHdrLv=[]
            self.lMarkDownFN=[]
            self.dMarkDown={}
            self.dImgFiles={}
            self.dStatus={
                'oky':{},
                'err':{},
                'skp':{},
                'wrn':{},
                }
            self.oFileBld=None
            # ----- end:
        except:
            self.logTB()
            return -1
    def _tpl(self,iVerbose=-1):
        """
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            sOrg='::'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,iVerbose))
            iRet=0
            # ----- end:
            # +++++ beg:
            # ----- end:
            # +++++ beg:
            self.oLog.debug('end:%s iRet:%d'%(sOrg,iRet))
            # ----- end:
            return iRet
        except:
            self.logTB()
            return -1
    def bldDat(self,sDef,oRef=None,**kwargs):
        """check definition has been queues/processed already  
        ### parameter
            sDef    ... definition to check
            oRef    ... reference object
            kwargs  ... flexible keyword argument
        ### return
            oDat    ... okay marked for processing done
        """
        try:
            sOrg='pyGatherMD::bldDat'
            if self.iVerbose>0:
                self.oLog.debug('beg:%s sDef:%s oRef:%r kwargs:%r'%(sOrg,sDef,oRef,kwargs))
            oDat=kwargs.get('oDat',self)
            if self.iVerbose>0:
                self.oLog.debug('end:%s sDef:%s oDat:%r'%(sOrg,sDef,oDat))
            return oDat
        except:
            self.logTB()
            return None
    def chgLv(self,iHdrLvCur,iHdrLvNxt,iVerbose=-1):
        """change header level
        ### parameter
            iHdrLvCur   ...
            iHdrLcNxt   ...
            iVerbose    ... verbose level
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
            -11 ... header level mismatch
        """
        try:
            # +++++ beg:
            self.oLog.debug('beg:chgLv iVerbose:%d'%(iVerbose))
            iRet=0
            # ----- end:
            # +++++ beg:
            dLvNxt={
                'name':'',
                'iLv':iHdrLvNxt,
                }
            if len(self.lHdrLv)>0:
                dLvPrv=self.lHdrLv[-1]
                while dLvPrv['iLv']>=dLvNxt['iLv']:
                    del self.lHdrLv[-1]
                    if len(self.lHdrLv)>0:
                        dLvPrv=self.lHdrLv[-1]
                    else:
                        break
            if len(self.lHdrLv)>0:
                dLvPrv=self.lHdrLv[-1]
                if dLvPrv['iLv']==(dLvNxt['iLv']-1):
                    iRet=1
                else:
                    self.oLog.error('  hdr lv mismatch prv:%d nxt:%d'%(dLvPrv['iLv'],dLvNxt['iLv']))
                    iRet=-11
            self.lHdrLv.append(dLvNxt)
            # ----- end:
            # +++++ beg:
            self.oLog.debug('end:chgLv  iRet:%d'%(iRet))
            # ----- end:
            return iRet
        except:
            logging.error(traceback.format_exc())
            return -1
    def loadCfg(self,sCfgFN='pyGatherMDCfg.json'):
        """read configuration file, given in json format
        ### parameter
            sCfgDN  ... configuration file name
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            sOrg='pyGatherMD::loadCfg'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            iRet=0
            self.__initCfg__()
            # ----- end:initialize
            # +++++ beg:read cfg file
            self.sCfgFN=sCfgFN
            with open(self.sCfgFN,'r') as oFile:
                self.dCfg=json.loads(oFile.read())
            # ----- end:read cfg file
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            return -1
    def validateDatPrm(self,sInFN='README.md',
                  sBldDN='./build',
                  sOtFN=None):
        """validate data parameter
        ### parameter
            sInFN   ... file name to process
            sBldDN  ... build directory name
            sOtFN   ... output file name
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:initialize
            sOrg='pyGatherMD::validateDatPrm'
            if self.iVerbose>0:
                self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
                self.oLog.debug('  sInFN:%r'%(sInFN))
                self.oLog.debug(' sBldDN:%r'%(sBldDN))
                self.oLog.debug('  sOtFN:%r'%(sOtFN))
            iRet=0
            self.__initDat__()
            # ----- end:initialize
            # +++++ beg:check call parameters
            sTmpInDN,sTmpInFN=os.path.split(sInFN)
            if self.iVerbose>5:
                self.oLog.debug('  sInFN:%r -> DN:%r FN:%r'%(sInFN,sTmpInDN,sTmpInFN))
            if sTmpInDN is None:
                sTmpInDN='./'
            if sOtFN is None:
                if sBldDN is None:
                    self.oLog.error('no output filename provided, no build directory given')
                    self.oLog.error('input file will not be overwritten, abort')
                    return -10
                sBldFN=os.path.join(sBldDN,sTmpInFN)
            else:
                if sBldDN is None:
                    if sTmpInFN==sOtFN:
                        self.oLog.error('output filename provided, no build directory given')
                        self.oLog.error('input file will not be overwritten, abort')
                        return -11
                    sTmpOtDN,sTmpOtFN=os.path.split(sOtFN)
                    if sTmpOtDN is None:
                        sTmpOtDN='./'
                    sBldDN=sTmpOtDN#os.path.join(sTmpInDN,sOtFN)
                    sBldFN=sOtFN#sTmpOtDN
                else:
                    self.oLog.debug('  sOtFN:%r'%(sOtFN))
                    self.oLog.debug(' sBldDN:%r sOtFN:%r'%(sBldDN,sOtFN))
                    sBldFN=os.path.join(sBldDN,sOtFN)
            if sBldFN==sInFN:
                self.oLog.error('build FN:%r and input FN:%r identical'%(sBldFN,sInFN))
                self.oLog.error('input file will not be overwritten, abort')
                return -19
            # ----- end:check call parameters
            # +++++ beg:validate adjusted
            sAbsInFN=os.path.abspath(sInFN)
            sAbsOtFN=os.path.abspath(sBldFN)
            if sAbsInFN==sAbsOtFN:
                return -15
            if os.path.exists(sInFN):
                iRet=1
            self.sInFN=sInFN
            self.sBldDN=sBldDN
            self.sBldFN=sBldFN
            # ----- end:validate adjusted
            # +++++ beg:
            if self.iVerbose>0:
                self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:
            return iRet
        except:
            self.logTB()
            return -1
    def copyImage(self,sCurDN,sImgFN):
        """copy image
        ### parameter
            sCurDN  ... current directory name
            sImgFN  ... image file name
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
        """
        try:
            # +++++ beg:
            if sCurDN is None:
                sCurDN=self.sCurDN
            self.oLog.debug('beg:copyImage:%s sCurDN:%s'%(sImgFN,sCurDN))
            iRet=0
            # ----- end:
            # +++++ beg:
            if sImgFN.startswith('./'):
                sTmpDN,sTmpFN=os.path.split(sImgFN)
                sSrcImgDN=sCurDN+sTmpDN[1:]
                sDstImgDN=self.sBldDN+sTmpDN[1:]
                sImgFN=sTmpFN
            else:
                sSrcImgDN=sCurDN+'/'
                sDstImgDN=self.sBldDN
            self.oLog.debug('  sSrcDN:%s'%(sSrcImgDN))
            self.oLog.debug('  sDstDN:%s'%(sDstImgDN))
            sSrcFullFN=os.path.join(sSrcImgDN,sImgFN)
            sDstFullFN=os.path.join(sDstImgDN,sImgFN)
            self.oLog.debug('  sSrcFullFN:%s'%(sSrcFullFN))
            self.oLog.debug('  sDstFullFN:%s'%(sDstFullFN))
            if sSrcFullFN in self.dImgFiles:
                self.oLog.warning('file:%s already copied'%(sSrcFullFN))
                return 0
            # ----- end:
            # +++++ beg:
            if os.path.exists(sDstImgDN)==False:
                try:
                    os.makedirs(sDstImgDN)
                except:
                    self.logTB()
            # ----- end:
        except:
            self.logTB()
            return -1
        try:
            # +++++ beg:
            self.dImgFiles[sSrcFullFN]=sDstFullFN
            shutil.copyfile(sSrcFullFN,sDstFullFN)
            self.dStatus['oky'][sSrcFullFN]=sDstFullFN
            # ----- end:
            # +++++ beg:
            self.oLog.debug('end:copyImage:%s  iRet:%d'%(sImgFN,iRet))
            # ----- end:
            return iRet
        except:
            self.logTB()
            self.dStatus['err'][sSrcFullFN]='copy to >%s< failed'%(sDstFullFN)
            #self.dImgFiles[sSrcFullFN]
            return -2
    def walkMD(self,sInFN='README.md',
                  sBldDN='./build',
                  sOtFN=None):
        """
        ### parameter
            sInFN   ... file name to process
            sBldDN  ... build directory
            sOtFN   ... output file name
        ### return
            >0  ... okay processing done
            =0  ... okay nop
            <0  ... error
            -5  ... build directory does not exit
        """
        try:
            # +++++ beg:initialize
            sOrg='pyGatherMD::walkMD'
            self.oLog.debug('beg:%s iVerbose:%d'%(sOrg,self.iVerbose))
            self.oLog.debug('  sInFN:%r'%(sInFN))
            self.oLog.debug(' sBldDN:%r'%(sBldDN))
            self.oLog.debug('  sOtFN:%r'%(sOtFN))
            iRet=0
            self.lLineOut=[]
            self.lMarkerMD=[]
            # ----- end:initialize
            # +++++ beg:validate parameter
            iRet=self.validateDatPrm(sInFN=sInFN,
                            sBldDN=sBldDN,
                            sOtFN=sOtFN)
            self.oLog.debug('    validateDatPrm iRet:%d'%(iRet))
            if iRet<0:
                logging.debug('end:%s  iRet:%d'%(sOrg,iRet))
                return iRet
            self.oLog.debug('  sInFN:%r'%(self.sInFN))
            self.oLog.debug(' sBldDN:%r'%(self.sBldDN))
            self.oLog.debug('  sOtFN:%r'%(self.sOtFN))
            # ----- end:validate parameter
            # +++++ beg:create build DN
            if os.path.exists(self.sBldDN)==False:
                try:
                    os.makedirs(self.sBldDN)
                except:
                    self.logTB()
            if os.path.exists(self.sBldDN)==False:
                self.oLog.error("end:%s return -5 sBldDN:%s "
                                "directory does not exist, "
                                "can't be created"%(sOrg,sBldDN))
                return -5
            # ----- end:create build DN
            # +++++ beg:create build file
            self.oLog.info(' sBldFN:%r'%(self.sBldFN))
            if self.sEnc is None:
                self.oFileBld=open(self.sBldFN,'w')
            else:
                self.oFileBld=open(self.sBldFN,'w',encoding=self.sEnc)
            # ----- end:create build file
            # +++++ beg:memorize file name
            sAbsInFN=os.path.abspath(self.sInFN)
            self.sCurDN,self.sCurFN=os.path.split(sAbsInFN)
            if self.iVerbose>5:
                self.oLog.debug('  sCurDN:%r sCurFN:%r'%(self.sCurDN,self.sCurFN))
            if self.sCurDN is None:
                self.sCurDN='./'
            if len(self.sCurDN)==0:
                self.sCurDN='./'
            # ----- end:memorize file name
            # +++++ beg:determine base file name to skip
            sTmpDN,sTmpFN=os.path.split(sAbsInFN)
            if sTmpDN is None:
                iSkipDN=-1
            else:
                iSkipDN=len(sTmpDN)+1
            # ----- end:determine base file name to skip
            # +++++ beg:process input file
            oMD=ldmMD(sAbsInFN,sLogger='oMD',iSkipDN=iSkipDN,
                      iVerbose=self.iVerbose)
            #oMD.prcBeg(sAbsInFN,oDat=oMD,oRef=None)
            oMD.prcBeg(sAbsInFN,oRef=None)
            oMD.prcExc(oGtrMD=self)
            iRetMD=oMD.prcEnd()
            # ----- end:process input file
            # +++++ beg:write output
            for iLine,sLine in oMD.lLine:
                self.oFileBld.write(sLine)
            # ----- end:write output
            # +++++ beg:close output
            if self.oFileBld is not None: 
                self.oFileBld.close()
            # ----- end:close output
            # +++++ beg:finalize
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,iRet))
            # ----- end:finalize
            return iRet
        except:
            self.logTB()
            self.oLog.debug('end:%s  iRet:%d'%(sOrg,-1))
            return -1

def execMain(sInFN='README.md',
                sBldDN='./build',
                sOtFN=None,
                sCfgFN=None,
                iVerbose=0):
    """main function to perform conversion
    """
    try:
        # +++++ beg:
        # ----- end:
        # +++++ beg:
        logging.debug('beg:execMain')
        iRet=0
        # ----- end:
        # +++++ beg:
        oGather=pyGatherMD(iVerbose=iVerbose)
        iRet=oGather.loadCfg(sCfgFN)
        iRet=oGather.walkMD(sInFN=sInFN,
                  sBldDN=sBldDN,
                  sOtFN=sOtFN)
        # ----- end:
        # +++++ beg:
        logging.debug('end:execMain iRet:%d'%(iRet))
        # ----- end:
        return iRet
    except:
        logging.error(traceback.format_exc())
        return -1

def main(args=None):
    # +++++ beg:
    # ----- end:
    
    # +++++ beg:init
    iVerbose=5                                          # 20190624 wro:set default verbose level
    # ----- end:init
    # +++++ beg:define CLI arguments
    usage = "usage: %prog [options]"
    oParser=OptionParser(usage,version="%prog "+__version__)
    oParser.add_option('-c','--cfgFN',dest='sCfgFN',
            default='pyGatherMDCfg.json',
            help='configuration file',metavar='pyGatherMDCfg.json',
            )
    oParser.add_option('-i','--inFN',dest='sInFN',
            default='README.md',
            help='markdown file to start',metavar='README.md',
            )
    oParser.add_option('-b','--buildDN',dest='sBldDN',
            default='./build',
            help='build directory',metavar='./build',
            )
    oParser.add_option('-o','--otFN',dest='sOtFN',
            default=None,
            help='output file',metavar='sng.md',
            )
    oParser.add_option('-l','--log',dest='sLogFN',
            default='./log/pyGatherMD.log',
            help='log filename',metavar='./log/pyGatherMD.log')
    oParser.add_option("-v", action="store_true", dest="verbose", default=True)
    oParser.add_option("-q", action="store_false", dest="verbose", default=True)
    # ----- end:define CLI arguments
    # +++++ beg:parse command line
    (opt,args)=oParser.parse_args(args=args)
    if opt.verbose:
        print("config FN:     %r"%(opt.sCfgFN))
        print(" input FN:     %r"%(opt.sInFN))
        print(" build DN:     %r"%(opt.sBldDN))
        print("output FN:     %r"%(opt.sOtFN))
        iVerbose=20
    # ----- end:parse command line
    # +++++ beg:prepare logging
    import lindworm.logUtil as logUtil
    logUtil.logInit(opt.sLogFN,iLevel=logging.DEBUG)
    # ----- end:prepare logging
    # +++++ beg:status
    print('logging file:%r'%(opt.sLogFN))
    # ----- end:status
    # +++++ beg:perform main action
    iRet=execMain(sInFN=opt.sInFN,
                sBldDN=opt.sBldDN,
                sOtFN=opt.sOtFN,
                sCfgFN=opt.sCfgFN,
                iVerbose=iVerbose)
    # ----- end:perform main action
    # +++++ beg:finished
    print('\nend processing')
    print('   iRet:%r'%(iRet))
    # ----- end:finished

if __name__=='__main__':
    # +++++ beg:call entry point
    main(args=None)
    # ----- end:call entry point
